/*  lil-gp Genetic Programming System, version 1.0, 11 July 1995
 *  Copyright (C) 1995  Michigan State University
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public License as
 *  published by the Free Software Foundation.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  
 *  Douglas Zongker       (zongker@isl.cps.msu.edu)
 *  Dr. Bill Punch        (punch@isl.cps.msu.edu)
 *
 *  Computer Science Department
 *  A-714 Wells Hall
 *  Michigan State University
 *  East Lansing, Michigan  48824
 *  USA
 *  
 */

#include <stdio.h>
#include <math.h>
#include "xm_lib.h"
#include "forms.h"
#include "gp-form.h"
#include "cache.h"
#include "auto-raters.h"
#include <lilgp.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>

#define UNSPECIFIED 0
#define INVALID     1
#define SIMPLE      2
#define LIST        3
#define TOURNAMENT  4
#define AUTO_LIST   5
#define AUTO_TOURN  6

Melody *melArray=NULL;     /* stores the current melodies */
int *val_array=NULL;       /* stores indexes during tournament sort */
int *tempArray=NULL;       /* used in the tournament sort */
int **wasComped=NULL;      /* stores comparison values for tournament sort.
			      At any location, (wasComped[i])[j] a:
			          '0' means i and j haven't been compared
				  '-1, -2' mean i<j, or i<<j
				  '1, 2' mean i>j, or i>>j
				  '4' means i==j */
int comparisons=0;         /* comparisons is the number of comparisons so
			      far for the tournament sort */
int expectedComps=0;       /* the number of comparisons probably needed */
int interface;        /* the xforms interface to use */

FILE *ratingLog; /* stores ratings for each generation */
FILE *comparisonLog; /* tournament comparison results */

function sets[4][50] =

{
  /*** RPB ***/
  { 

    /*** Functions ***/
    { f_play_two, NULL, NULL, 2, "play-two", FUNC_DATA, -1, 0 },
    { f_add_space, NULL, NULL, 1, "add-space", FUNC_DATA, -1, 0 },
    { f_play_twice, NULL, NULL, 1, "play-twice", FUNC_DATA, -1, 0 },
    { f_shift_up, NULL, NULL, 1, "shift-up", FUNC_DATA, -1, 0 },
    { f_shift_down, NULL, NULL, 1, "shift-down", FUNC_DATA, -1, 0 },
    { f_mirror, NULL, NULL, 1, "mirror", FUNC_DATA, -1, 0 },
    { f_play_and_mirror, NULL, NULL, 1, "play-and-mirror", FUNC_DATA, -1, 0 },
/*  { NULL, NULL, NULL, -1, "ADF0", EVAL_DATA, 1, 0 },
    { NULL, NULL, NULL, -1, "ADF1", EVAL_DATA, 2, 0 },
    { NULL, NULL, NULL, -1, "ADF2", EVAL_DATA, 3, 0 },
*/
    /*** Terminals ***/
    { f_C_4, NULL, NULL, 0, "C-4", TERM_NORM, -1, 0 },
    { f_D_4, NULL, NULL, 0, "D-4", TERM_NORM, -1, 0 },
    { f_E_4, NULL, NULL, 0, "E-4", TERM_NORM, -1, 0 },
    { f_F_4, NULL, NULL, 0, "F-4", TERM_NORM, -1, 0 },
    { f_G_4, NULL, NULL, 0, "G-4", TERM_NORM, -1, 0 },
    { f_A_5, NULL, NULL, 0, "A-5", TERM_NORM, -1, 0 },
    { f_B_5, NULL, NULL, 0, "B-5", TERM_NORM, -1, 0 },
    { f_RST, NULL, NULL, 0, "RST", TERM_NORM, -1, 0 }
#ifndef C_MAJOR
    ,{ f_CS4, NULL, NULL, 0, "C#4", TERM_NORM, -1, 0 },
    { f_DS4, NULL, NULL, 0, "D#4", TERM_NORM, -1, 0 },
    { f_FS4, NULL, NULL, 0, "F#4", TERM_NORM, -1, 0 },
    { f_GS4, NULL, NULL, 0, "G#4", TERM_NORM, -1, 0 },
    { f_AS5, NULL, NULL, 0, "A#5", TERM_NORM, -1, 0 }
#endif
    /* second octave */
    , { f_C_5, NULL, NULL, 0, "C-5", TERM_NORM, -1, 0 },
    { f_D_5, NULL, NULL, 0, "D-5", TERM_NORM, -1, 0 },
    { f_E_5, NULL, NULL, 0, "E-5", TERM_NORM, -1, 0 },
    { f_F_5, NULL, NULL, 0, "F-5", TERM_NORM, -1, 0 },
    { f_G_5, NULL, NULL, 0, "G-5", TERM_NORM, -1, 0 },
    { f_A_6, NULL, NULL, 0, "A-6", TERM_NORM, -1, 0 },
    { f_B_6, NULL, NULL, 0, "B-6", TERM_NORM, -1, 0 }
#ifndef C_MAJOR
    , { f_CS5, NULL, NULL, 0, "C#5", TERM_NORM, -1, 0 },
    { f_DS5, NULL, NULL, 0, "D#5", TERM_NORM, -1, 0 },
    { f_FS5, NULL, NULL, 0, "F#5", TERM_NORM, -1, 0 },
    { f_GS5, NULL, NULL, 0, "G#5", TERM_NORM, -1, 0 },
    { f_AS6, NULL, NULL, 0, "A#6", TERM_NORM, -1, 0 }
#endif 

#ifdef CHORDS
    , { f_C_Chord, NULL, NULL, 0, "C-Chord", TERM_NORM, -1, 0 },
    { f_D_Chord, NULL, NULL, 0, "D-Chord", TERM_NORM, -1, 0 },
    { f_E_Chord, NULL, NULL, 0, "E-Chord", TERM_NORM, -1, 0 },
    { f_F_Chord, NULL, NULL, 0, "F-Chord", TERM_NORM, -1, 0 },
    { f_G_Chord, NULL, NULL, 0, "G-Chord", TERM_NORM, -1, 0 },
    { f_A_Chord, NULL, NULL, 0, "A-Chord", TERM_NORM, -1, 0 },
    { f_B_Chord, NULL, NULL, 0, "B-Chord", TERM_NORM, -1, 0 }
#endif
    /* second octave chords */
    , { f_C2_Chord, NULL, NULL, 0, "C2-Chord", TERM_NORM, -1, 0 },
    { f_D2_Chord, NULL, NULL, 0, "D2-Chord", TERM_NORM, -1, 0 },
    { f_E2_Chord, NULL, NULL, 0, "E2-Chord", TERM_NORM, -1, 0 },
    { f_F1_Chord, NULL, NULL, 0, "F1-Chord", TERM_NORM, -1, 0 },
    { f_G1_Chord, NULL, NULL, 0, "G1-Chord", TERM_NORM, -1, 0 },
    { f_A1_Chord, NULL, NULL, 0, "A1-Chord", TERM_NORM, -1, 0 },
    { f_B1_Chord, NULL, NULL, 0, "B1-Chord", TERM_NORM, -1, 0 }

  },

  /*** ADF0 ***/
  { 

    /*** Functions ***/
    { f_play_two, NULL, NULL, 2, "play-two", FUNC_DATA, -1, 0 },
    { f_add_space, NULL, NULL, 1, "add-space", FUNC_DATA, -1, 0 },
    { f_play_twice, NULL, NULL, 1, "play-twice", FUNC_DATA, -1, 0 },
    { f_shift_up, NULL, NULL, 1, "shift-up", FUNC_DATA, -1, 0 },
    { f_shift_down, NULL, NULL, 1, "shift-down", FUNC_DATA, -1, 0 },
    { f_mirror, NULL, NULL, 1, "mirror", FUNC_DATA, -1, 0 },
    { f_play_and_mirror, NULL, NULL, 1, "play-and-mirror", FUNC_DATA, -1, 0 },
    
    /*** Terminals ***/
    { f_C_4, NULL, NULL, 0, "C-4", TERM_NORM, -1, 0 },
    { f_D_4, NULL, NULL, 0, "D-4", TERM_NORM, -1, 0 },
    { f_E_4, NULL, NULL, 0, "E-4", TERM_NORM, -1, 0 },
    { f_F_4, NULL, NULL, 0, "F-4", TERM_NORM, -1, 0 },
    { f_G_4, NULL, NULL, 0, "G-4", TERM_NORM, -1, 0 },
    { f_A_5, NULL, NULL, 0, "A-5", TERM_NORM, -1, 0 },
    { f_B_5, NULL, NULL, 0, "B-5", TERM_NORM, -1, 0 },
    { f_RST, NULL, NULL, 0, "RST", TERM_NORM, -1, 0 }
#ifndef C_MAJOR
    ,{ f_CS4, NULL, NULL, 0, "C#4", TERM_NORM, -1, 0 },
    { f_DS4, NULL, NULL, 0, "D#4", TERM_NORM, -1, 0 },
    { f_FS4, NULL, NULL, 0, "F#4", TERM_NORM, -1, 0 },
    { f_GS4, NULL, NULL, 0, "G#4", TERM_NORM, -1, 0 },
    { f_AS5, NULL, NULL, 0, "A#5", TERM_NORM, -1, 0 }
#endif

#ifdef CHORDS
    , { f_C_Chord, NULL, NULL, 0, "C-Chord", TERM_NORM, -1, 0 },
    { f_D_Chord, NULL, NULL, 0, "D-Chord", TERM_NORM, -1, 0 },
    { f_E_Chord, NULL, NULL, 0, "E-Chord", TERM_NORM, -1, 0 },
    { f_F_Chord, NULL, NULL, 0, "F-Chord", TERM_NORM, -1, 0 },
    { f_G_Chord, NULL, NULL, 0, "G-Chord", TERM_NORM, -1, 0 },
    { f_A_Chord, NULL, NULL, 0, "A-Chord", TERM_NORM, -1, 0 },
    { f_B_Chord, NULL, NULL, 0, "B-Chord", TERM_NORM, -1, 0 }
#endif

  },

  /*** ADF1 ***/
  {
    /*** Functions ***/
    { f_play_two, NULL, NULL, 2, "play-two", FUNC_DATA, -1, 0 },
    { f_add_space, NULL, NULL, 1, "add-space", FUNC_DATA, -1, 0 },
    { f_play_twice, NULL, NULL, 1, "play-twice", FUNC_DATA, -1, 0 },
    { f_shift_up, NULL, NULL, 1, "shift-up", FUNC_DATA, -1, 0 },
    { f_shift_down, NULL, NULL, 1, "shift-down", FUNC_DATA, -1, 0 },
    { f_mirror, NULL, NULL, 1, "mirror", FUNC_DATA, -1, 0 },
    { f_play_and_mirror, NULL, NULL, 1, "play-and-mirror", FUNC_DATA, -1, 0 },

    /*** Terminals ***/
      { f_C_4, NULL, NULL, 0, "C-4", TERM_NORM, -1, 0 },
    { f_D_4, NULL, NULL, 0, "D-4", TERM_NORM, -1, 0 },
    { f_E_4, NULL, NULL, 0, "E-4", TERM_NORM, -1, 0 },
    { f_F_4, NULL, NULL, 0, "F-4", TERM_NORM, -1, 0 },
    { f_G_4, NULL, NULL, 0, "G-4", TERM_NORM, -1, 0 },
    { f_A_5, NULL, NULL, 0, "A-5", TERM_NORM, -1, 0 },
    { f_B_5, NULL, NULL, 0, "B-5", TERM_NORM, -1, 0 },
    { f_RST, NULL, NULL, 0, "RST", TERM_NORM, -1, 0 }
#ifndef C_MAJOR
    ,{ f_CS4, NULL, NULL, 0, "C#4", TERM_NORM, -1, 0 },
    { f_DS4, NULL, NULL, 0, "D#4", TERM_NORM, -1, 0 },
    { f_FS4, NULL, NULL, 0, "F#4", TERM_NORM, -1, 0 },
    { f_GS4, NULL, NULL, 0, "G#4", TERM_NORM, -1, 0 },
    { f_AS5, NULL, NULL, 0, "A#5", TERM_NORM, -1, 0 }
#endif 

#ifdef CHORDS
    , { f_C_Chord, NULL, NULL, 0, "C-Chord", TERM_NORM, -1, 0 },
    { f_D_Chord, NULL, NULL, 0, "D-Chord", TERM_NORM, -1, 0 },
    { f_E_Chord, NULL, NULL, 0, "E-Chord", TERM_NORM, -1, 0 },
    { f_F_Chord, NULL, NULL, 0, "F-Chord", TERM_NORM, -1, 0 },
    { f_G_Chord, NULL, NULL, 0, "G-Chord", TERM_NORM, -1, 0 },
    { f_A_Chord, NULL, NULL, 0, "A-Chord", TERM_NORM, -1, 0 },
    { f_B_Chord, NULL, NULL, 0, "B-Chord", TERM_NORM, -1, 0 }
#endif

    
  },

  /*** ADF2 ***/
  {
    /*** Functions ***/
    { f_play_two, NULL, NULL, 2, "play-two", FUNC_DATA, -1, 0 },
    { f_add_space, NULL, NULL, 1, "add-space", FUNC_DATA, -1, 0 },
    { f_play_twice, NULL, NULL, 1, "play-twice", FUNC_DATA, -1, 0 },
    { f_shift_up, NULL, NULL, 1, "shift-up", FUNC_DATA, -1, 0 },
    { f_shift_down, NULL, NULL, 1, "shift-down", FUNC_DATA, -1, 0 },
    { f_mirror, NULL, NULL, 1, "mirror", FUNC_DATA, -1, 0 },
    { f_play_and_mirror, NULL, NULL, 1, "play-and-mirror", FUNC_DATA, -1, 0 },
    { NULL, NULL, NULL, -1, "ADF0", EVAL_DATA, 1, 0 },
    { NULL, NULL, NULL, -1, "ADF1", EVAL_DATA, 2, 0 },
    
    /*** Terminals ***/
    { f_C_4, NULL, NULL, 0, "C-4", TERM_NORM, -1, 0 },
    { f_D_4, NULL, NULL, 0, "D-4", TERM_NORM, -1, 0 },
    { f_E_4, NULL, NULL, 0, "E-4", TERM_NORM, -1, 0 },
    { f_F_4, NULL, NULL, 0, "F-4", TERM_NORM, -1, 0 },
    { f_G_4, NULL, NULL, 0, "G-4", TERM_NORM, -1, 0 },
    { f_A_5, NULL, NULL, 0, "A-5", TERM_NORM, -1, 0 },
    { f_B_5, NULL, NULL, 0, "B-5", TERM_NORM, -1, 0 },
    { f_RST, NULL, NULL, 0, "RST", TERM_NORM, -1, 0 }
#ifndef C_MAJOR
    ,{ f_CS4, NULL, NULL, 0, "C#4", TERM_NORM, -1, 0 },
    { f_DS4, NULL, NULL, 0, "D#4", TERM_NORM, -1, 0 },
    { f_FS4, NULL, NULL, 0, "F#4", TERM_NORM, -1, 0 },
    { f_GS4, NULL, NULL, 0, "G#4", TERM_NORM, -1, 0 },
    { f_AS5, NULL, NULL, 0, "A#5", TERM_NORM, -1, 0 }
#endif 

#ifdef CHORDS
    , { f_C_Chord, NULL, NULL, 0, "C-Chord", TERM_NORM, -1, 0 },
    { f_D_Chord, NULL, NULL, 0, "D-Chord", TERM_NORM, -1, 0 },
    { f_E_Chord, NULL, NULL, 0, "E-Chord", TERM_NORM, -1, 0 },
    { f_F_Chord, NULL, NULL, 0, "F-Chord", TERM_NORM, -1, 0 },
    { f_G_Chord, NULL, NULL, 0, "G-Chord", TERM_NORM, -1, 0 },
    { f_A_Chord, NULL, NULL, 0, "A-Chord", TERM_NORM, -1, 0 },
    { f_B_Chord, NULL, NULL, 0, "B-Chord", TERM_NORM, -1, 0 }
#endif

    
  }
  
  
};

/* app_build_function_sets()
 *
 * this function should build data structures describing the function
 * set(s) and the trees of the individual.  it should then pass these
 * structures to function_sets_init(), destroy its data structures,
 * and return the (integer) return value from function_sets_init().
 *
 * see the manual for more information on constructing this function.
 */
int app_build_function_sets ( void )
{
     function_set *fset;
     int *tree_map;
     char **tree_name;
     int ret;

     int num_function_sets = 1;
     int num_trees = 1;

     fset = (function_set *)MALLOC ( num_function_sets *
                                    sizeof ( function_set ) );
     fset[0].size = 32; 
#ifdef C_MAJOR
     fset[0].size -= 10;
#endif

#ifdef CHORDS
     fset[0].size += 14;
#endif
     
     fset[0].cset = sets[0];

     /*
     fset[1].size = 20; 
     fset[1].cset = sets[1];

     fset[2].size = 21; 
     fset[2].cset = sets[2];

     fset[3].size = 22; 
     fset[3].cset = sets[3];
*/
     
     tree_map = (int *)MALLOC ( num_trees * sizeof ( int ) );

     /* specify, for each tree, what function set to use. */
     tree_map[0] = 0;
  /*   tree_map[1] = 1;
     tree_map[2] = 2;
     tree_map[3] = 3;
*/
     tree_name = (char **)MALLOC ( num_trees * sizeof ( char * ) );

     /* give each tree a name. */
     tree_name[0] = "MAIN";
/*     tree_name[1] = "ADF0";
     tree_name[2] = "ADF1";
     tree_name[3] = "ADF2";
  */   
     ret = function_sets_init ( fset, 1, tree_map,
                               tree_name, 1 );

     FREE ( fset );
     FREE ( tree_map );
     FREE ( tree_name );

     return ret;
}

/*
 * lessThan - returns 1 if melArray[index1]'s melody is rated lower than
 *   melArray[index2]'s melody.  This is achieved by looking in the
 *   wasComped array to see if the comparison has been made previously.
 *   If it hasn't the user shown the form and asked to rate the pieces.
 */
int lessThan(int index1, int index2) {

  globaldata *g;
  char textlabel[30];
  FD_Tourn *fd_Tourn;
  int value;

  double rating1, rating2, delta;
  
  /* see if we need to get the rating */
  if(!((wasComped[index1])[index2])) {

    /* if we are using the neural net, just figure out the comparison */
    if(interface==AUTO_TOURN) {

      if(tAutoRate(melArray[index1].noteSequence,
		   melArray[index2].noteSequence) < 0.5)
	(wasComped[index1])[index2]=-1;
      else
	(wasComped[index1])[index2]=1;
   
    }  else {
    
      /* get the interface */
      g=get_globaldata();
      fd_Tourn=g->fd_Tourn;
      
      /* write the comparisons label */
      sprintf(textlabel, "%d/~%d", comparisons, expectedComps);
      fl_set_object_label(fd_Tourn->Ind_Number, textlabel);
      
      /* set the slider to 0 */
      fl_set_slider_value(fd_Tourn->Ind_Rater, 0);
      
      /* attach the noteSequences to the buttons */
      fd_Tourn->Play_Button_1->u_vdata=(void *)(melArray[index1].noteSequence);
      fd_Tourn->Play_Button_2->u_vdata=(void *)(melArray[index2].noteSequence);
      
      /* Number the buttons */
      fd_Tourn->Play_Button_1->u_ldata=1;
      fd_Tourn->Play_Button_2->u_ldata=2;
      
      /* put the song listings in the boxes */
      fl_clear_browser(fd_Tourn->Song1_Browser);
      fl_add_browser_line(fd_Tourn->Song1_Browser, \
			  melArray[index1].melodyString);
      fl_clear_browser(fd_Tourn->Song2_Browser);
      fl_add_browser_line(fd_Tourn->Song2_Browser, \
			  melArray[index2].melodyString);
      
      /* Play the two songs */
      fl_freeze_form(fd_Tourn->Tourn);
      wait(NULL);
      if( vfork()  == 0 ) 
	execlp("vplay", "vplay", "-q", "melody1.wav", 0x0);
      wait(NULL);
      fl_call_object_callback(fd_Tourn->Play_Button_1);
      wait(NULL);
      
      if( vfork() == 0 ) 
	execlp("vplay", "vplay", "-q", "melody2.wav", 0x0);
      wait(NULL);
      fl_call_object_callback(fd_Tourn->Play_Button_2);
      fl_unfreeze_form(fd_Tourn->Tourn);
      
      /* now activate the form */
      if (fd_Tourn->Exit_Button == fl_do_forms()) exit(0);
      
      /* see what the rating is */
      value=(int)fl_get_slider_value(fd_Tourn->Ind_Rater);
      
      if(value==0) {
	(wasComped[index1])[index2]=4;
	(wasComped[index2])[index1]=4;
      } else {
	(wasComped[index1])[index2]=-value;
	(wasComped[index2])[index1]=value;
      }
      
    }
  
    /* increase the comparisons counter */
    comparisons++;
  
  }

  value=(wasComped[index1])[index2];

  if(value>0)
    return 0;
  if(value<0)
     return 1;

  printf("Error: can't get here!\n");

  return -1;
  
}

/*
 * mergeSort - sorts the melodies in the val_array using the merge sort
 *   technique.  lessThan is called for comparisons.
 */
void mergeSort(int ms_low_ind, int ms_high_ind) {

  int median;
  int temp;
  int i,j,k;
    
  /* if one element, return */
  if(ms_low_ind >= ms_high_ind) {

    return;
    
  }

  /* if two elements, swap if needed */
  if( (ms_high_ind - ms_low_ind)==1) {

    /* swap if misordered */
    if(lessThan(val_array[ms_high_ind], val_array[ms_low_ind])) {
      temp=val_array[ms_high_ind];
      val_array[ms_high_ind]=val_array[ms_low_ind];
      val_array[ms_low_ind]=temp;
    }

    return;
    
  }

  /* otherwise, recursively sort the two halves */
  median=(ms_high_ind + ms_low_ind) /2 ;
  mergeSort(ms_low_ind, median);
  mergeSort(median+1, ms_high_ind);

  /* Now merge them */
  for(i=ms_low_ind,j=median+1,k=0;(i<=median) && (j<=ms_high_ind);k++)
    /* if the first sections current lowest less than second sections, take
     it as the next lowest */
    if(lessThan(val_array[i],val_array[j]))
      tempArray[k]=val_array[i++];
    else
      /* take the second sections as the next lowest */
      tempArray[k]=val_array[j++];

  /* take those remaining in one of the sections, and add them to the end of
     the tempArray */
  if(i>median)
    while(j<=ms_high_ind)
      tempArray[k++]=val_array[j++];
  else
    while(i<=median)
      tempArray[k++]=val_array[i++];

  /* copy tempArray values into val_array */
  for(i=ms_low_ind,k=0;i<=ms_high_ind;i++,k++)
    val_array[i]=tempArray[k];

  /* now return */
  return;
   
}

/* app_group_eval_fitness()
 *
 * this function looks at all of the individuals in the array and rates them
 * all at once.  It places the ratings into the melodyArray, which then
 * assigns the ratings to each individual when app_eval_fitness is called for
 * that individual.
 *
 * NOTE: We could probably do this in one step, but I am trying to minimize
 *       hacking to the main body of the lil-gp kernel code
 */
void app_group_eval_fitness(individual *allInds, int numInds)
{

  int i,j;
  DATATYPE noteSequence;
  globaldata *g;
  FILE *xmFile;
  char textlabel[30];
  FD_List *fd_List;
  FL_OBJECT *theBrowser;
  int maxInd;
  double tournRating;
  double spread;

  g=get_globaldata();
  
  /* evaluate all the individuals and fill in the melArray */
  for(i=0;i<numInds;i++) {
    
    set_current_individual ( allInds+i );
  
    melArray[i].noteSequence=evaluate_tree ( (allInds+i)->tr[0].data, 0 );;
    melArray[i].melodyString=sprintNoteSequence(melArray[i].noteSequence);

    melArray[i].rating=queryCache(melArray[i].noteSequence);
    if(melArray[i].rating!=-1) {
      melArray[i].isRated=1;
      melArray[i].isSuggestion=1;
    } else {
      melArray[i].rating=1;
      melArray[i].isRated=0;
      melArray[i].isSuggestion=0;
    }
  }

  if(interface==AUTO_LIST) {

    for(i=0;i<numInds;i++)
      melArray[i].rating=100*lAutoRate(melArray[i].noteSequence);
    
  } else if(interface==SIMPLE) {
    
    /* Now do GUI rating stuff */
    for(i=0;i<numInds;i++) {
      
      /*printf("\nCurrent Melody:\n\n%s\n", melArray[i].melodyString);
       */
      
      xmFile=fopen("pat.bin","w+");
      
      fwriteXMPatternInfo(xmFile, melArray[i].noteSequence);
      
      /* printf("Pattern Information:\n");
	 fseek(xmFile, 0, SEEK_SET);
	 fprintXMPatternInfo(xmFile, stdout, 4, 1);
	 */
      
      fclose(xmFile);
      
      /*
	pretty_print_individual(ind, stdout);
	
	*/	 
      
      /* Print the individual number to the xforms box */
      sprintf(textlabel, "%d/%d", i, (numInds-1));
      fl_set_object_label(g->fd_Main->Ind_Number, textlabel);
            
      fl_call_object_callback(g->fd_Main->Play_Button); 
      
      if (g->fd_Main->Exit_Button == fl_do_forms()) exit(0);
      
      melArray[i].rating=fl_get_slider_value(g->fd_Main->Ind_Rate_Slider);
      melArray[i].isRated=1;
      melArray[i].isSuggestion=0;
      
    }

    /* end of SIMPLE interface section */
    
  } else if(interface==LIST) {

    /* get the interface from global storage */
    fd_List=g->fd_List;
    theBrowser=fd_List->Indiv_Browser;
    maxInd=((ListerInfo *)(fd_List->vdata))->maxInd;

    /* freeze the form for modifications */
    fl_freeze_form(fd_List->List);
    
    /* for each Melody put it into the Melody list box */
    for(i=0;i<maxInd;i++)
      set_melody(theBrowser, &(melArray[i]), i);
    
    /* Now display the first Melody */
    fl_select_browser_line(theBrowser, 1);
    fl_call_object_callback(theBrowser);
    
    /* unfreeze the list form */
    fl_unfreeze_form(fd_List->List);

    /* play the first song for the generation */
    fl_call_object_callback(fd_List->Play_Button);

    /* let the user interact, but make sure the generation is completely
       rated */
    do {
      
      if(fl_do_forms() == fd_List->Next_Gen_Button) {
	
	for(i=0;i<maxInd;i++)
	  if(melArray[i].isRated == 0) {
	    fl_show_alert("Error: ","All Melodies must be rated!!","",1);
	    break;
	  }
	
	if(i==maxInd) break;
	
      } else exit(1);

    } while(1);
    
    /* end of LIST interface section */
    
  } else if(interface==TOURNAMENT || interface==AUTO_TOURN) {

    /* initialize the arrays */
    for(i=0;i<numInds;i++) {
      val_array[i]=i;
      for(j=0;j<numInds;j++)
	(wasComped[i])[j]=0;
    }
    comparisons=0;
    
    /* now sort the array */
    mergeSort(0,numInds-1);

    /* now assign ratings based on the sort */
    spread=40/(double)numInds; /* distributes ratings from 10 to 90 */
    for(i=0,tournRating=10;i<numInds-1;i++) {

      melArray[val_array[i]].rating=tournRating;
      melArray[val_array[i]].isRated=1;
      melArray[val_array[i]].isSuggestion=0;

      if((wasComped[val_array[i+1]])[val_array[i]]!=4)
	tournRating+=spread*(double)(wasComped[val_array[i+1]])[val_array[i]];
    }
    
    melArray[val_array[i]].rating=tournRating;
    melArray[val_array[i]].isRated=1;
    melArray[val_array[i]].isSuggestion=0;

    /*
     * bring up the list interface to display tournament results
     */
    if(interface==TOURNAMENT) {

      /* get the interface from global storage */
      fd_List=g->fd_List;
      theBrowser=fd_List->Indiv_Browser;
      maxInd=((ListerInfo *)(fd_List->vdata))->maxInd;
      
      /* for each Melody put it into the Melody list box */
      for(i=0;i<maxInd;i++)
	set_melody(theBrowser, &(melArray[i]), i);
      
      /* Now display the first Melody */
      fl_select_browser_line(theBrowser, 1);
      fl_call_object_callback(theBrowser);
      
      /* hide tournament form, show list form */
      fl_hide_form(g->fd_Tourn->Tourn);
      fl_show_form(fd_List->List, FL_PLACE_CENTER, \
		   FL_FULLBORDER,"GP-Music Tournament Results");
      
      /* activate the list form */
      if(fl_do_forms() == fd_List->Exit_Button) exit(1);
      
      /* hide list form, show tournament form */
      fl_hide_form(fd_List->List);
      fl_show_form(g->fd_Tourn->Tourn,FL_PLACE_CENTER,FL_FULLBORDER,"GP-Music "
		   "Comparator");
    }
    /* end of TOURNAMENT interface section */
  } else {
    
    printf("\nInvalid interface!\n\n");
    exit(3);

  }
  
  /* cache, store and free all strings in melArray */
  for(i=0;i<numInds;i++) {

    size_t temp;
    
    /* store size of sequence */
    temp=strlen(melArray[i].noteSequence)+1;
    fwrite(&temp, sizeof(size_t), 1, ratingLog);
  
    /* store note sequence */
    fwrite(melArray[i].noteSequence, sizeof(char), temp, ratingLog);

    /* store rating */
    fwrite(&(melArray[i].rating), sizeof(double), 1, ratingLog);

    /* update cache */
    updateCache(melArray[i].noteSequence, melArray[i].rating);

    /* if tournament interface, write the same info to the comparisonLog */
    if(interface==TOURNAMENT) {
      fwrite(&temp, sizeof(size_t), 1, comparisonLog);
      fwrite(melArray[i].noteSequence, sizeof(char), temp, comparisonLog);
      fwrite(&(melArray[i].rating), sizeof(double), 1, comparisonLog);
    }
    
    /* free entries */
    FREE(melArray[i].noteSequence);
    FREE(melArray[i].melodyString);
    melArray[i].noteSequence=NULL;
    melArray[i].melodyString=NULL;
        
  }
  
  /* if tournament interface, write out this generations comparisons */
  if(interface==TOURNAMENT)
    for(i=0;i<numInds;i++)
      fwrite(wasComped[i], sizeof(int), numInds, comparisonLog);
      


} /* end of app_group_eval_fitness */

/* app_eval_fitness()
 *
 * this function should evaluate the fitness of the individual.  typically
 * this function will loop over all the fitness cases.  the following
 * fields in the (individual *) should be filled in:
 *    r_fitness    (raw fitness)
 *    s_fitness    (standardized fitness)
 *    a_fitness    (adjusted fitness)
 *    hits         (hits)
 *    evald        (always set to EVAL_CACHE_VALID)
 */

void app_eval_fitness ( individual *ind , int indNum)
{

  /* just look up fitness from the group eval */
  ind->r_fitness = melArray[indNum].rating;
  /*  printf("Rating for individual %d was %lg\n", indNum, ind->r_fitness);
   */

  if(interface==AUTO_TOURN || interface==AUTO_LIST)
    printf("\tIndividual %d rated %g\n", indNum, ind->r_fitness);
  
  ind->hits = 1;
  
  /* compute the standardized and raw fitness. */
  ind->s_fitness = 100 - ind->r_fitness;
  ind->a_fitness = 1/(1+ind->s_fitness);
  
  /* always leave this line in. */
  ind->evald = EVAL_CACHE_VALID;
  
}

/* app_end_of_evaluation()
 *
 * this is called every generation after the entire popualation has
 * been evaluated.  "mpop" is a pointer to basically everything --
 * every individual in every subpopulation, exchange topology for
 * multipop runs, etc.  see the documentation for useful things to
 * find inside it.
 *
 * run_stats[0].best[0]->ind is always a pointer to the best-of-run
 * individual.  if the "newbest" flag is 1, it indicates that this
 * is a NEW best-of-run individual.
 *
 * similarly, gen_stats[0].best[0]->ind points to the best-of-generation
 * individual.
 *
 * DO NOT MODIFY ANY OF THE STRUCTURES YOU ARE PASSED POINTERS TO.
 *
 * should return 1 to indicate that the termination criterion has been
 * met and the run should terminate, 0 otherwise.  you do not need to
 * test the generation number against the maximum -- this is done by
 * the kernel.
 *
 * you can dynamically modify breeding parameters and/or subpop exchange
 * topology by making the appropriate changes to the parameter database.
 * see the manual for more information.  after making changes, you need
 * to call:
 *    rebuild_breeding_table ( mpop );
 * or
 *    rebuild_exchange_topology ( mpop );
 * for your changes to take effect.
 */
 
int app_end_of_evaluation ( int gen, multipop *mpop, int newbest,
                           popstats *gen_stats, popstats *run_stats )
{
  FILE *xmFile;
  char *noteSequence;
  char filename[100];
  char copyCommand[100];
  
  /* get note sequence for best of generation individual */
  noteSequence = evaluate_tree ( gen_stats[0].best[0]->ind->tr[0].data, 0 );
  
  /* write XM pattern for best individual in this generation */
  sprintf(filename,"best-gen-%02d.bin",gen);
  xmFile=fopen(filename,"w+");
  
  fwriteXMPatternInfo(xmFile, noteSequence);

  fclose(xmFile);

  FREE(noteSequence);
  
  /* if this is the new best overall, copy it to the overall pattern file */
  if(newbest) {

    sprintf(copyCommand,"cp best-gen-%02d.bin best-run.bin",gen);
    system(copyCommand);
    
  }
  
  return 0;
}

/* app_end_of_breeding()
 *
 * this is called every generation after the next population is created
 * (but before it is evaluated).
 *
 * DO NOT MODIFY ANY OF THE STRUCTURES YOU ARE PASSED POINTERS TO.
 */

void app_end_of_breeding ( int gen, multipop *mpop )
{
     return;
}

/* app_create_output_streams()
 *
 * if you are going to create any custom output streams, do it here.
 * this is not documented yet, look at the source for the symbolic
 * regression problem for a guide.
 *
 * return 0 if init went OK and the run can proceed, or 1 to abort.
 */

int app_create_output_streams()
{
     return 0;
}

/*
 * init_simple_interface - initializes the application to use the simple
 *   xforms interface to rate melodies
 */
void init_simple_interface() {

  globaldata *g;

  /* get the global data structure */
  g=get_globaldata();

  g->fd_Main = create_form_Main();
  
  /* fill-in form initialization code */
  fl_set_object_label(g->fd_Main->Gen_Number, "0/5");
  fl_set_object_label(g->fd_Main->Ind_Number, "0/16");
  
  /* show the first form */
  fl_show_form(g->fd_Main->Main,FL_PLACE_CENTER,FL_FULLBORDER,"GP Music");

} /* end of init_simple_interface */

/*
 * init_list_interface - initializes the application to use the list
 *   xforms interface to rate melodies
 */
void init_list_interface(int maxInd) {

  globaldata *g;
  FD_List *fd_List;
  FL_OBJECT *theBrowser;
  int i;
  
  /* get the global data structure and point it to the interface */
  g=get_globaldata();
  fd_List = create_form_List();
  g->fd_List=fd_List;
  
  /* initialize the ListerInfo */
  fd_List->vdata=(void *)malloc(sizeof(ListerInfo));
  ((ListerInfo *)(fd_List->vdata))->melArray=melArray;
  ((ListerInfo *)(fd_List->vdata))->maxInd=maxInd;
  ((ListerInfo *)(fd_List->vdata))->currentInd=0;
  
  /*
   * initialize the form and other stuff
   */
  theBrowser=fd_List->Indiv_Browser;
  
  /* set the browser, rater and play button objects to point back
     to the main form */
  theBrowser->u_vdata=(void *)fd_List;
  fd_List->Ind_Rate_Slider->u_vdata=(void *)fd_List;
  fd_List->Play_Button->u_vdata=(void *)fd_List;
  fd_List->Prev_Button->u_vdata=(void *)fd_List;
  fd_List->Next_Button->u_vdata=(void *)fd_List;
  fd_List->Inc_Rater->u_vdata=(void *)fd_List;
  fd_List->Dec_Rater->u_vdata=(void *)fd_List;
  
  /* put an entry in text box for each Melody */
  for(i=0;i<maxInd;i++)
    fl_add_browser_line(theBrowser, "Melody ??");
  
} /* end of init_list_interface */

/*
 * init_tournament_interface - initializes the application to use the
 *   tournament xforms interface to rate melodies
 */
void init_tournament_interface(int maxInd, int maxGen) {

  globaldata *g;
  FD_Tourn *fd_Tourn;
  int i;

  comparisonLog=fopen("comparison.log","w+");
  fwrite(&maxGen, sizeof(int), 1, comparisonLog);  /* store number of gens */
  fwrite(&maxInd, sizeof(int), 1, comparisonLog);  /* store number of inds */

  /* get the global data structure and point it to the interface */
  g=get_globaldata();
  fd_Tourn = create_form_Tourn();
  g->fd_Tourn=fd_Tourn;
  
  fd_Tourn->Inc_Button->u_vdata=(void *)fd_Tourn;
  fd_Tourn->Dec_Button->u_vdata=(void *)fd_Tourn;

  /* initialize the arrays */
  val_array=(int *)MALLOC(maxInd*sizeof(int));
  tempArray=(int *)MALLOC(maxInd*sizeof(int));
  wasComped=(int **)MALLOC(maxInd*sizeof(int *));
  for(i=0;i<maxInd;i++)
    wasComped[i]=(int *)MALLOC(maxInd*sizeof(int));

  /* set the number of expected comparisons per generation */
  expectedComps=0.9*(double)maxInd*log((double)maxInd)/0.69314718 - 
    0.6*(double)maxInd-3.2;
  
} /* end of init_tournament_interface */

/* app_initialize()
 *
 * this should perform any application-specific initialization the user
 * needs to do.
 *
 * return 0 if init went OK and the run can proceed, or 1 to abort.
 */

int app_initialize ( int startfromcheckpoint, int argc, char **argv , \
		     int maxInd)
{
  int i;
  char *interfaceType;
  int maxGen;
  int dummy;
  
  /* open the ratings log file */
  ratingLog=fopen("rating.log","w+");
  maxGen=atoi( get_parameter("max_generations") );
  fwrite(&maxGen, sizeof(int), 1, ratingLog);  /* store number of gens */
  fwrite(&maxInd, sizeof(int), 1, ratingLog);  /* store number of inds */

  /* allocate and initialize the melody array */
  melArray=(Melody *)MALLOC(maxInd*sizeof(Melody));

  for(i=0;i<maxInd;i++) {
    melArray[i].noteSequence=NULL;
    melArray[i].melodyString=NULL;
    melArray[i].rating=1;
    melArray[i].isRated=0;
    melArray[i].isSuggestion=0;
  }

  /* initialize the cache */
  initCache( maxGen, maxInd);
  
  /* figure out which interface is requested */
  interfaceType=get_parameter("interface");

  if(interfaceType != NULL) {

    interface=INVALID;
    if(strcmp(interfaceType, "simple")==0) interface=SIMPLE;
    if(strcmp(interfaceType, "list")==0) interface=LIST;
    if(strcmp(interfaceType, "tournament")==0) interface=TOURNAMENT;
    if(strcmp(interfaceType, "auto-list")==0) interface=AUTO_LIST;
    if(strcmp(interfaceType, "auto-tourn")==0) interface=AUTO_TOURN;

  } else {

    interface=UNSPECIFIED;
    printf("\nInterface not specified, using list interface.\n\n");

  }  

  switch(interface) {

  case SIMPLE: printf("\nSimple interface chosen!\n\n"); break;
  case LIST: printf("\nList interface chosen!\n\n"); break;
  case TOURNAMENT: printf("\nTournament interface chosen!\n\n"); break;
  case AUTO_LIST: printf("\nList auto-rater chosen!\n\n"); break;
  case AUTO_TOURN: printf("\nTournament auto-rater chosen!\n\n"); break;
  case INVALID:
    printf("\nInvalid interface specified!\n\n");
    exit(3);
    break;
  default: printf("\nImpossible to get here!\n\n"); exit(3); break;

  }

  /* initialize Xforms */
  fl_initialize(&argc, argv, "GPMusic", 0, 0);

  /* Now initialize the appropriate interface */
  switch (interface) {

  case SIMPLE:
    init_simple_interface();
    break;
  case LIST:
  case UNSPECIFIED:
    init_list_interface(maxInd);
    fl_show_form(get_globaldata()->fd_List->List, FL_PLACE_CENTER, \
		 FL_FULLBORDER,"GP-Music Generation Rater");
    break;
  case TOURNAMENT:
    init_tournament_interface(maxInd, maxGen);
    fl_show_form(get_globaldata()->fd_Tourn->Tourn,
		 FL_PLACE_CENTER,FL_FULLBORDER,"GP-Music Comparator");
    init_list_interface(maxInd);
    fl_hide_object(get_globaldata()->fd_List->Ind_Rate_Slider);
    break;
  case AUTO_LIST:
    InitListRater(get_parameter("weightfile"));
    break;
  case AUTO_TOURN:
    init_tournament_interface(maxInd, maxGen);
    InitTournRater(get_parameter("weightfile"));
    break;    
  default:
    printf("Interface problem, exiting\n");
    exit(3);
    break;
  }

  return 0;

}

/* app_uninitialize()
 *
 * perform application cleanup (free memory, etc.)
 */

void app_uninitialize ( void )
{
  int maxInd;
  int i;
  
  maxInd=atoi(get_parameter("pop_size"));
  
  /* deallocate the melody array */
  for(i=0;i<maxInd;i++) {

    FREE(melArray[i].noteSequence);
    FREE(melArray[i].melodyString);

  }

  FREE(melArray);

  /* if tournament interface, deallocate its arrays */
  if(interface==TOURNAMENT) {
    FREE(val_array);
    for(i=0;i<maxInd;i++)
      FREE(wasComped[i]);
    FREE(wasComped);
  }
      
  /* close the cache */
  closeCache();

  /* destroy the neural nets */
  if(interface==AUTO_LIST || interface==AUTO_TOURN)
    uninitRaters();
  
  /* close the rating log */
  fclose(ratingLog);
  
  return;
  
}

/* app_read_checkpoint()
 *
 * read state information from a checkpoint file.  it is passed a handle
 * for a file that has been opened for read in text mode.  it should
 * leave the file pointer at the end of the user information.
 */

void app_read_checkpoint ( FILE *f )
{
     return;
}

/* app_write_checkpoint()
 *
 * write state information to a checkpoint file.  it is passed a handle
 * for a file that has been opened for write in text mode.  it should
 * leave the file pointer at the end of the user information.
 */

void app_write_checkpoint ( FILE *f )
{
     return;
}

/*
 * app_set_gen - notifies the application of a new generation so it can
 *   set the dialog boxes appropriately
 */
void
app_set_gen(int gen, int maxgen) {

  globaldata *g;
  char textlabel[30];

  g=get_globaldata();
  
  sprintf(textlabel, "%d/%d", gen, maxgen);

  switch(interface) {

  case SIMPLE:
    fl_set_object_label(g->fd_Main->Gen_Number, textlabel);
    break;
  case TOURNAMENT:
    fl_set_object_label(g->fd_Tourn->Gen_Number, textlabel);
  case LIST:
    fl_set_object_label(g->fd_List->Gen_Number, textlabel);
    break;
  case AUTO_TOURN:
  case AUTO_LIST:
    break;
  default:
    printf("\nUnsupported interface!\n\n");
    exit(3);
    
  }
    
}

/*
 * app_abort_indiv - called during the breeding of new generations, and
 *   during creation of the initial population.  If '1' is returned for the
 *   provided individual, it is aborted out of the population, and a new
 *   individual is bred to replace it.
 */
int
app_abort_indiv(tree *aTree) {

  DATATYPE noteSequence;
  int seqLength;
  int minSequence=1;
  int maxSequence=255;
  int minNotes=1;
  int maxNotes=255;
  int numNotes=0;
  char *parameter;
  int i;
  
  /* see if abortions are active */
  if((parameter=get_parameter("abortions"))==NULL) return 0;
  if(!atoi(parameter)) return 0;

  /* now get abortion parameters */
  if((parameter=get_parameter("min_sequence")) != NULL)
    minSequence=atoi(parameter);  
  if((parameter=get_parameter("max_sequence")) != NULL)
    maxSequence=atoi(parameter);
  if((parameter=get_parameter("min_notes")) != NULL)
    minNotes=atoi(parameter);
  if((parameter=get_parameter("max_notes")) != NULL)
    maxNotes=atoi(parameter);

  /* now evaluate the individuals tree */
  noteSequence = evaluate_tree ( aTree[0].data, 0 );

  /* get the sequence length */
  seqLength=strlen(noteSequence);

  /* count the number of notes in sequence */
  for(i=0,numNotes=0;i<seqLength;i++)
    if(noteSequence[i]!=88) numNotes++;

  /* free the note sequence */
  FREE(noteSequence);
  
  if(seqLength<=minSequence || seqLength>maxSequence ||
     numNotes<=minNotes || numNotes>maxNotes) {

    /*    if(seqLength<=minSequence)
      printf("Individual shorter than %d: Aborted!\n", minSequence);
    else if(seqLength>maxSequence)
      printf("Individual longer than %d: Aborted!\n", maxSequence);
    else if(numNotes<=minNotes)
      printf("Individual has less than %d notes: Aborted!\n", minNotes);
    else
      printf("Individual has more than %d notes: Aborted!\n", maxNotes);
      */
    return 1;

  } else
    return 0;
  
}
    

