#ifndef _cache_h_
#define _cache_h_

/*
 * cache.h cache.c - these files cache old noteSequences and their ratings.
 *   The cache can be polled to see if a noteSequence has been rated before,
 *   and if so, what it was rated.
 */

/*
 * initCache - initializes the cache for a run with numGen generations and
 *   numInd individuals per generation
 */
void
initCache(int numGen, int numInd);

/*
 * resetCache - clears the current cached entries
 */
void
resetCache();

/*
 * closeCache - deallocates all memory used by the cache.  The cache must
 *   then be reinitialized before it can be used again.
 */
void
closeCache();

/*
 * queryCache - checks to see if the noteSequence has been rated before.  If
 *   so, the returned value is the previous rating.  Otherwise the value
 *   returned is -1.
 */
double
queryCache(const char *noteSequence);

/*
 * updateCache - notifies the cache that the given noteSequence has been
 *   assigned the given new rating.  The noteSequence need not have been
 *   in the cache before.
 */
void
updateCache(const char *noteSequence, double rating);


#endif /* _cache_h_ */





