/*  lil-gp Genetic Programming System, version 1.0, 11 July 1995
 *  Copyright (C) 1995  Michigan State University
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public License as
 *  published by the Free Software Foundation.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  
 *  Douglas Zongker       (zongker@isl.cps.msu.edu)
 *  Dr. Bill Punch        (punch@isl.cps.msu.edu)
 *
 *  Computer Science Department
 *  A-714 Wells Hall
 *  Michigan State University
 *  East Lansing, Michigan  48824
 *  USA
 *  
 */

#ifndef _FUNCTIONS_H
#define _FUNCTIONS_H

#include <lilgp.h>

/* here you should place prototypes for all the C functions implementing
 * functions and terminals, as well as those generating and printing
 * ephemeral random constants.  this should look like:
 *
 *  
 *  DATATYPE f_function ( int tree, farg *args );
 *     (for ordinary functions and terminals)
 *
 *  void f_ephem_gen ( DATATYPE * );
 *     (for C functions that generate random constants)
 *
 *  char *f_ephem_print ( DATATYPE );
 *     (for C functions that print random constants to strings)
 */

/* functions (all data) */
DATATYPE f_play_two ( int tree, farg *args );
DATATYPE f_add_space ( int tree, farg *args );
DATATYPE f_play_twice ( int tree, farg *args );
DATATYPE f_shift_up (int tree, farg *args );
DATATYPE f_shift_down ( int tree, farg *args );
DATATYPE f_mirror ( int tree, farg *args );
DATATYPE f_play_and_mirror (int tree, farg *args);

/* terminals */
DATATYPE f_C_4 ( int tree, farg *args );
DATATYPE f_CS4 ( int tree, farg *args );
DATATYPE f_D_4 ( int tree, farg *args );
DATATYPE f_DS4 ( int tree, farg *args );
DATATYPE f_E_4 ( int tree, farg *args );
DATATYPE f_F_4 ( int tree, farg *args );
DATATYPE f_FS4 ( int tree, farg *args );
DATATYPE f_G_4 ( int tree, farg *args );
DATATYPE f_GS4 ( int tree, farg *args );
DATATYPE f_A_5 ( int tree, farg *args );
DATATYPE f_AS5 ( int tree, farg *args );
DATATYPE f_B_5 ( int tree, farg *args );
DATATYPE f_C_5 ( int tree, farg *args );
DATATYPE f_CS5 ( int tree, farg *args );
DATATYPE f_D_5 ( int tree, farg *args );
DATATYPE f_DS5 ( int tree, farg *args );
DATATYPE f_E_5 ( int tree, farg *args );
DATATYPE f_F_5 ( int tree, farg *args );
DATATYPE f_FS5 ( int tree, farg *args );
DATATYPE f_G_5 ( int tree, farg *args );
DATATYPE f_GS5 ( int tree, farg *args );
DATATYPE f_A_6 ( int tree, farg *args );
DATATYPE f_AS6 ( int tree, farg *args );
DATATYPE f_B_6 ( int tree, farg *args );
DATATYPE f_RST ( int tree, farg *args );

/* pseudo chords */
DATATYPE f_F1_Chord ( int tree, farg *args );
DATATYPE f_G1_Chord ( int tree, farg *args );
DATATYPE f_A1_Chord ( int tree, farg *args );
DATATYPE f_B1_Chord ( int tree, farg *args );
DATATYPE f_C_Chord ( int tree, farg *args );
DATATYPE f_D_Chord ( int tree, farg *args );
DATATYPE f_E_Chord ( int tree, farg *args );
DATATYPE f_F_Chord ( int tree, farg *args );
DATATYPE f_G_Chord ( int tree, farg *args );
DATATYPE f_A_Chord ( int tree, farg *args );
DATATYPE f_B_Chord ( int tree, farg *args );
DATATYPE f_C2_Chord ( int tree, farg *args );
DATATYPE f_D2_Chord ( int tree, farg *args );
DATATYPE f_E2_Chord ( int tree, farg *args );

#endif




