#include "forms.h"
#include "gp-form.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>

/* callbacks for forms Main and List*/
/* callbacks for form List */
void ind_selected(FL_OBJECT *obj, long data)
{

  FD_List *fd_List;
  ListerInfo *sequenceInfo;
  int indNumber, maxInd;
  char tempString[30];
  
  /* retrieve the main form and the sequenceInfo*/
  fd_List=(FD_List *)obj->u_vdata;
  sequenceInfo=(ListerInfo *)(fd_List->vdata);
  maxInd=sequenceInfo->maxInd;

  /* freeze the form */
  fl_freeze_form(fd_List->List);
  
  /* get individual number */
  indNumber=fl_get_browser(obj)-1;
  sequenceInfo->currentInd=indNumber;
  
  /* set individual number counter */
  sprintf(tempString, "%d/%d", indNumber, (sequenceInfo->maxInd-1));
  fl_set_object_label(fd_List->Ind_Number, tempString);
     
  /* display sequence info */
  fl_clear_browser(fd_List->Seq_Text);
  fl_add_browser_line(fd_List->Seq_Text, \
		      sequenceInfo->melArray[indNumber].melodyString);
  
  /* set rating */
  fl_set_slider_value(fd_List->Ind_Rate_Slider, \
		      sequenceInfo->melArray[indNumber].rating);
  
  /* if we are now at melody 0, make Prev_Button look inactive, and
   Next_Button active */
  if(indNumber==0) {
    fl_set_object_lcolor(fd_List->Prev_Button,FL_INACTIVE);
    fl_set_object_lcolor(fd_List->Next_Button,FL_BLACK);
  }
  
  /* else if we are now at melody (maxInd-1), make Next_Button look inactive,
   and Prev_Button active */
  else if(indNumber==(maxInd-1)) {
    fl_set_object_lcolor(fd_List->Next_Button,FL_INACTIVE);
    fl_set_object_lcolor(fd_List->Prev_Button,FL_BLACK);
  }

  /* otherwise we are somewhere in the middle, so make both look active */
  else {
    fl_set_object_lcolor(fd_List->Next_Button,FL_BLACK);
    fl_set_object_lcolor(fd_List->Prev_Button,FL_BLACK);
  }

  /* unfreeze the form */
  fl_unfreeze_form(fd_List->List);
  
  return;
  
}

void ind_rater(FL_OBJECT *obj, long data)
{

  FD_List *fd_List;
  ListerInfo *sequenceInfo;
  int indNumber;
  char tempString[30];
  Melody *melArray;
  double newRating;
  
  /* retrieve the main form and the sequenceInfo*/
  fd_List=(FD_List *)obj->u_vdata;
  sequenceInfo=(ListerInfo *)(fd_List->vdata);
  melArray=sequenceInfo->melArray;
  indNumber=sequenceInfo->currentInd;

  /* find out the new rating */
  newRating=fl_get_slider_value(obj);
    
  /* Update the melArray to reflect new rating */
  melArray[indNumber].rating=newRating;
  melArray[indNumber].isRated=1;
  melArray[indNumber].isSuggestion=0;

  /* Update the sequence browser to reflect new rating */
  set_melody(fd_List->Indiv_Browser, &(melArray[indNumber]), indNumber);
  
}

void play_indiv(FL_OBJECT *ob, long data)
{
  static pid_t last_xm_pid=0;
  static pid_t last_wav_pid=0;
  char commandString[40];
  FILE *xmFile;
  FD_List *fd_List;
  ListerInfo *sequenceInfo;
  int indNumber;
  Melody *melArray;
    
  fd_List=(FD_List *)ob->u_vdata;

  /* wait for last process to terminate */
  if(last_xm_pid !=0) {
    waitpid(last_xm_pid, NULL, 0);
    last_xm_pid=0;
  }

  /* if data == 1, we are using the list interface, so... */
  if(data==1) {

    /* retrieve the main form and the sequenceInfo*/
    sequenceInfo=(ListerInfo *)(fd_List->vdata);
    melArray=sequenceInfo->melArray;
    indNumber=sequenceInfo->currentInd;
         
    /* create the XM pattern data for the selected info */
    xmFile=fopen("pat.bin","w+");
      
    fwriteXMPatternInfo(xmFile, melArray[indNumber].noteSequence);
    
    fclose(xmFile);

    /*printf("Creating new XM for melody %d.\n", indNumber);
     */

    /* we are using the tournament selector */
  } else if(data==2) {

    /* create the XM pattern data for the selected info */
    xmFile=fopen("pat.bin","w+");
      
    fwriteXMPatternInfo(xmFile, (DATATYPE)(ob->u_vdata));
    
    fclose(xmFile);

  }

  if( ( last_xm_pid = vfork() ) == 0 )
      execlp("play_pattern", "play_pattern", "pat.bin", 0x0);
  
}

/* selects the individual delta before current in the list */
void delta_select_ind(FL_OBJECT *obj, long delta) {

  FD_List *fd_List;
  ListerInfo *sequenceInfo;
  int indNumber, maxInd;
  double newRating;

  /* if delta is 0, do nothing */
  if(delta==0) return;

  /* retrieve the main form and the sequenceInfo*/
  fd_List=(FD_List *)obj->u_vdata;
  sequenceInfo=(ListerInfo *)(fd_List->vdata);
  indNumber=sequenceInfo->currentInd;
  maxInd=sequenceInfo->maxInd;

  /* freeze the form */
  fl_freeze_form(fd_List->List);
  
  /* See if we are at melody 0 and act appropriately */
  if(indNumber==0)
    if (delta < 0) {

      /* unfreeze the form */
      fl_unfreeze_form(fd_List->List);
      
      return;
    }
    else fl_set_object_lcolor(fd_List->Prev_Button,FL_BLACK);
  
  /* See if we are at melody (maxInd-1), and act appropriately */
  if(indNumber==(maxInd-1))
    if( delta > 0) {
      
      /* unfreeze the form */
      fl_unfreeze_form(fd_List->List);
      
      return;
    }
    else fl_set_object_lcolor(fd_List->Next_Button,FL_BLACK);

  /* change to the new melody */
  indNumber+=delta;
  fl_select_browser_line(fd_List->Indiv_Browser, indNumber+1);
  fl_call_object_callback(fd_List->Indiv_Browser);
  sequenceInfo->currentInd=indNumber;

  /* call the play button to play the new melody */
  fl_call_object_callback(fd_List->Play_Button);
  
  /* if we are now at melody 0, make Prev_Button look inactive */
  if(indNumber==0)
    fl_set_object_lcolor(fd_List->Prev_Button,FL_INACTIVE);
  
  /* if we are now at melody (maxInd-1), make Next_Button look inactive */
  if(indNumber==(maxInd-1))
     fl_set_object_lcolor(fd_List->Next_Button,FL_INACTIVE);
     
  /* unfreeze the form */
  fl_unfreeze_form(fd_List->List);
  
}

void set_melody(FL_OBJECT *browser, Melody *ind, int indNumber) {

  char tempString[50];
  
  if (ind->isSuggestion)
    sprintf(tempString, "Melody %d\t[%lg]", indNumber, ind->rating);
  else if (ind->isRated)
    sprintf(tempString, "Melody %d\t(%lg)", indNumber, ind->rating);
  else
    sprintf(tempString, "Melody %d", indNumber);

  fl_replace_browser_line(browser, indNumber+1, tempString);
       
}

void delta_rater(FL_OBJECT *obj, long delta) {

  FD_List *fd_List;
  FL_OBJECT *rater;
  double currentRating;
  /* if delta is 0, do nothing */
  if(delta==0) return;

  /* retrieve the main form and the slider */
  fd_List=(FD_List *)obj->u_vdata;
  rater=fd_List->Ind_Rate_Slider;

  /* get the current rating*/
  currentRating=fl_get_slider_value(rater);

  /* set a new value, if appropriate */
  currentRating+=delta;
  if(currentRating>0 && currentRating<=100)
    fl_set_slider_value(rater, currentRating);

  /* call the callback for the slider */
  fl_call_object_callback(rater);

}

void delta_tourn_rater(FL_OBJECT *obj, long delta)
{

  FD_Tourn *fd_Tourn;
  FL_OBJECT *rater;
  double currentRating;
  /* if delta is 0, do nothing */
  if(delta==0) return;

  /* retrieve the main form and the slider */
  fd_Tourn=(FD_Tourn *)obj->u_vdata;
  rater=fd_Tourn->Ind_Rater;

  /* get the current rating*/
  currentRating=fl_get_slider_value(rater);

  /* set a new value, if appropriate */
  currentRating+=delta;
  if(currentRating>=-2 && currentRating<=2)
    fl_set_slider_value(rater, currentRating);

}

void dummy(FL_OBJECT *obj, long delta) {

  /* do nothing */
}





