#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define max(xVal,yVal) ( (xVal) > (yVal) ? (xVal) : (yVal) )

/* stop criterion types */
#define AVG 0
#define MAX 1
#define ITER 2

/* network comparison typ */
#define BINARY 0
#define COMPLEX 1

/* different plot data that can be generated */
#define NONE 0
#define T_SUM_ERRORS 1
#define C_SUM_ERRORS 2
#define T_VS_C_SUM_ERRORS 3
#define T_VS_C_PERCENT_CORRECT 6
#define T_LENGTH_HISTOGRAM 7
#define C_LENGTH_HISTOGRAM 8
#define T_ERROR_DISTRIB 9
#define C_ERROR_DISTRIB 10

/* the network weight arrays */
double *aOneWeights;
double aOneBias;
double *aTwoWeights;
double aTwoBias;
double *aThreeWeights;
double aThreeBias;
double *aOutputWeights;

double *bOneWeights;
double bOneBias;
double *bTwoWeights;
double bTwoBias;
double *bThreeWeights;
double bThreeBias;
double *bOutputWeights;

double outputBias;		  

/*
 * Defines the level spreads for the tournament auto rater
 *
 *   The spread of the top level integrator will be adjusted to accept
 *   the maximum number of inputs from level 3 given the maximimum sequence
 *   length is T_MAX_LENGTH.
 */
double T_ONE_SPREAD=8;
double T_ONE_DELTA=4;
double T_TWO_SPREAD=8;
double T_TWO_DELTA=4;
double T_THREE_SPREAD=4;
double T_THREE_DELTA=2;
double T_MAX_LENGTH=0;

/*
 * initNetVals - allocates the weight arrays
 */
void initNetVals() {

  double outputLength;
  
  /* create the weight and bias arrays */
  aOneWeights=(double *)malloc(T_ONE_SPREAD*sizeof(double));
  aTwoWeights=(double *)malloc(T_TWO_SPREAD*sizeof(double));
  aThreeWeights=(double *)malloc(T_THREE_SPREAD*sizeof(double));

  bOneWeights=(double *)malloc(T_ONE_SPREAD*sizeof(double));
  bTwoWeights=(double *)malloc(T_TWO_SPREAD*sizeof(double));
  bThreeWeights=(double *)malloc(T_THREE_SPREAD*sizeof(double));

  outputLength=max(1,ceil(1+(T_MAX_LENGTH-T_ONE_SPREAD)/T_ONE_DELTA));
  outputLength=max(1,ceil(1+(outputLength-T_TWO_SPREAD)/T_TWO_DELTA));
  outputLength=max(1,ceil(1+(outputLength-T_THREE_SPREAD)/T_THREE_DELTA));

  aOutputWeights=(double *)malloc(outputLength*sizeof(double));
  bOutputWeights=(double *)malloc(outputLength*sizeof(double));

  outputBias=( drand48() > 0.5 ? 1 : -1)*drand48();

} /* end of initNetVals */

/*
 * freadNetState - reads all of the networks weights and biases from
 *   the given file.  If writeWeights is true, the weights and parameters
 *   are dumped to the file "tourn-weights.data"
 */
void freadNetState(FILE *outputFile, int writeWeights) {

  double outputLength;
  FILE *weightFile;
  
  fread(aOneWeights, sizeof(double), T_ONE_SPREAD, outputFile);
  fread(&aOneBias, sizeof(double), 1, outputFile);

  fread(bOneWeights, sizeof(double), T_ONE_SPREAD, outputFile);
  fread(&bOneBias, sizeof(double), 1, outputFile);

  fread(aTwoWeights, sizeof(double), T_TWO_SPREAD, outputFile);
  fread(&aTwoBias, sizeof(double), 1, outputFile);

  fread(bTwoWeights, sizeof(double), T_TWO_SPREAD, outputFile);
  fread(&bTwoBias, sizeof(double), 1, outputFile);

  fread(aThreeWeights, sizeof(double), T_THREE_SPREAD, outputFile);
  fread(&aThreeBias, sizeof(double), 1, outputFile);

  fread(bThreeWeights, sizeof(double), T_THREE_SPREAD, outputFile);
  fread(&bThreeBias, sizeof(double), 1, outputFile);

  outputLength=max(1,ceil(1+(T_MAX_LENGTH-T_ONE_SPREAD)/T_ONE_DELTA));
  outputLength=max(1,ceil(1+(outputLength-T_TWO_SPREAD)/T_TWO_DELTA));
  outputLength=max(1,ceil(1+(outputLength-T_THREE_SPREAD)/T_THREE_DELTA));

  fread(aOutputWeights, sizeof(double), (size_t)outputLength, outputFile);
  fread(bOutputWeights, sizeof(double), (size_t)outputLength, outputFile);
  fread(&outputBias, sizeof(double), 1, outputFile);

  if(writeWeights) {

    /* open the file */
    weightFile=fopen("tourn-weights.data", "w+");
  
    /* store the spread and delta values */
    fwrite(&T_ONE_SPREAD, sizeof(double), 1, weightFile);
    fwrite(&T_ONE_DELTA, sizeof(double), 1, weightFile);
    fwrite(&T_TWO_SPREAD, sizeof(double), 1, weightFile);
    fwrite(&T_TWO_DELTA, sizeof(double), 1, weightFile);
    fwrite(&T_THREE_SPREAD, sizeof(double), 1, weightFile);
    fwrite(&T_THREE_DELTA, sizeof(double), 1, weightFile);
    fwrite(&T_MAX_LENGTH, sizeof(double), 1, weightFile);
    
    /* store the weights and biases */
    fwrite(aOneWeights, sizeof(double), T_ONE_SPREAD, weightFile);
    fwrite(&aOneBias, sizeof(double), 1, weightFile);
    
    fwrite(bOneWeights, sizeof(double), T_ONE_SPREAD, weightFile);
    fwrite(&bOneBias, sizeof(double), 1, weightFile);
    
    fwrite(aTwoWeights, sizeof(double), T_TWO_SPREAD, weightFile);
    fwrite(&aTwoBias, sizeof(double), 1, weightFile);
    
    fwrite(bTwoWeights, sizeof(double), T_TWO_SPREAD, weightFile);
    fwrite(&bTwoBias, sizeof(double), 1, weightFile);
    
    fwrite(aThreeWeights, sizeof(double), T_THREE_SPREAD, weightFile);
    fwrite(&aThreeBias, sizeof(double), 1, weightFile);
    
    fwrite(bThreeWeights, sizeof(double), T_THREE_SPREAD, weightFile);
    fwrite(&bThreeBias, sizeof(double), 1, weightFile);
    
    fwrite(aOutputWeights, sizeof(double), (size_t)outputLength, weightFile);
    fwrite(bOutputWeights, sizeof(double), (size_t)outputLength, weightFile);
    fwrite(&outputBias, sizeof(double), 1, weightFile);
    
    /* close the file */
    fclose(weightFile);

  }
  
} /* end of freadNetState */

int main(int argc, char *argv[]) {

  int i,j;
  int iterations=0;

  double tMaxSumSqrErr;    /* errors for training set */
  double tAvgSumSqrErr;
  double tMinSumSqrErr;
  double tPercentCorrect;
  
  int *tLengthBuckets;
  double *tErrorBuckets;
  
  int tErrorDistrib[100];
    
  double cMaxSumSqrErr;    /* errors for control set */
  double cAvgSumSqrErr;
  double cMinSumSqrErr;
  double cPercentCorrect;
  
  int *cLengthBuckets;
  double *cErrorBuckets;

  int cErrorDistrib[100];
  
  double maxErr=0.02;
  int stopType=AVG;
  int outputMode=BINARY;
  double lrate=0.5;
  double momentum=0;
  long int seedVal;
  int checkFreq=100;
  char *statFileName=NULL;
  FILE *statFile=NULL;
  
  int trainingSetSize;
  int controlSetSize;

  int alphaType=0;
  int plotType=NONE;
  int plotIteration=0;
  int quietMode=0;

  FILE *dataFile=NULL;
  char *dataFileName=NULL;

  int plotMade=0;

  int writeWeights=1;

  double lastGood;
  
  /* init with command line params */
  for(i=1;i<argc;i++) {
    
    if(!strcmp("-alpha",argv[i])) {
      alphaType=1;
    } else if(!strcmp("-tSumErr",argv[i])) {
      plotType=T_SUM_ERRORS;
    } else if(!strcmp("-cSumErr",argv[i])) {
      plotType=C_SUM_ERRORS;
    } else if(!strcmp("-tcSumErr",argv[i])) {
      plotType=T_VS_C_SUM_ERRORS;
    } else if(!strcmp("-tcPerCor",argv[i])) {
      plotType=T_VS_C_PERCENT_CORRECT;
    } else if(!strcmp("-tLengthHist",argv[i])) {
      plotType=T_LENGTH_HISTOGRAM;
      plotIteration=atoi(argv[++i]);
    } else if(!strcmp("-cLengthHist",argv[i])) {
      plotType=C_LENGTH_HISTOGRAM;
      plotIteration=atoi(argv[++i]);
    } else if(!strcmp("-tErrorDist",argv[i])) {
      plotType=T_ERROR_DISTRIB;
      plotIteration=atoi(argv[++i]);
    } else if(!strcmp("-cErrorDist",argv[i])) {
      plotType=C_ERROR_DISTRIB;
      plotIteration=atoi(argv[++i]);
    } else if(!strcmp("-o",argv[i])) {
      dataFileName=argv[++i];
    } else if(!strcmp("-q",argv[i])) {
      quietMode=1;
    } else if(!strcmp("-h",argv[i])) {
      printf("\nSyntax: %s [-alpha] [-tSumErr|-cSumErr|-tcSumErr|"
	     "-tcPerCor\n\t\t|-tLengthHist <iteration>|-cLengthHist "
	     "<iteration>|"
	     "-tErrorDist\n\t\t<iteration>|-cErrorDist <iteration>] "
	     "[-q] [-o <Data File>]\n\t\t<Statistics File>\n\n"
	     "\t-alpha\t\t\tstat file generated on DEC Alpha\n"
	     "\t-tSumErr\t\tgenerate sum square error data for training set\n"
	     "\t-cSumErr\t\tgenerate sum square error data for control set\n"
	     "\t-tcSumErr\t\tgenerate training versus control set sum square"
	     "\n\t\t\t\terror data\n"
	     "\t-tcPerCor\t\tgenerate training versus control set percent"
	     "\n\t\t\t\tcorrect data\n"
	     "\t-tLengthHist\t\tgenerate training set decimal error vs "
	     "length\n\t\t\t\thistogram for the given iteration\n"
	     "\t-cLengthHist\t\tgenerate control set decimal error vs "
	     "length\n\t\t\t\thistogram for the given iteration\n"
	     "\t-tErrorDist\t\tgenerate training set decimal error "
	     "distribution\n\t\t\t\tdata for the given iteration\n"
	     "\t-cErrorDist\t\tgenerate control set decimal error "
	     "distribution\n\t\t\t\tdata for the given iteration\n"
	     "\t-q\t\t\tsupresses information for each iteration\n"
	     "\t-o\t\t\toutput data to specified data file\n\n"
	     "\tStatistics File\t\tstat file output by l-auto-rater\n\n",
	     argv[0]);
      exit(1);
    } else {
      statFileName=argv[i];
    }
    
  }

  /* read run parameters */
  printf("\nReading statistics from file '%s'\n", statFileName);
  
  statFile=fopen(statFileName, "r");

  /* print the plot type being used */
  if(plotType==NONE)
    printf("No output data being generated.\n\n");
  else
    printf("Generating data for a ");
  
  switch(plotType) {

  case NONE: break;
  case T_SUM_ERRORS: printf("training set sum error"); break;
  case C_SUM_ERRORS: printf("control set sum error"); break;
  case T_VS_C_SUM_ERRORS: printf("training and control set sum error"); break;
  case T_VS_C_PERCENT_CORRECT:
    printf("training and control set percent correct");
    break;
  case T_LENGTH_HISTOGRAM: printf("training set decimal error vs. length");
    break;
  case C_LENGTH_HISTOGRAM: printf("control set decimal error vs. length");
    break;
  case T_ERROR_DISTRIB: printf("training set decimal error distribution");
    break;
  case C_ERROR_DISTRIB: printf("control set decimal error distribution");
    break;

  default: printf("impossible");

  }
  
  if(plotType!=NONE)
    printf(" plot.\n\n");

  /* open output file */
  if(plotType!=NONE)
    if(dataFileName==NULL) {
      dataFile=fopen("output.data", "w+");
      printf("Outputting data to file 'output.data'.\n\n");
    } else {
      dataFile=fopen(dataFileName, "w+");
      printf("Outputting data to file '%s'.\n\n", dataFileName);
    }
  
  /* write out the run configuration */
  fread(&stopType, sizeof(int), 1, statFile);
  fread(&maxErr, sizeof(double), 1, statFile);
  fread(&lrate, sizeof(double), 1, statFile);
  fread(&seedVal, sizeof(long int), 1, statFile);
  if(alphaType) fseek(statFile, 4, SEEK_CUR);
  fread(&outputMode, sizeof(int), 1, statFile);
  fread(&T_ONE_SPREAD, sizeof(double), 1, statFile);
  fread(&T_ONE_DELTA, sizeof(double), 1, statFile);
  fread(&T_TWO_SPREAD, sizeof(double), 1, statFile);
  fread(&T_TWO_DELTA, sizeof(double), 1, statFile);
  fread(&T_THREE_SPREAD, sizeof(double), 1, statFile);
  fread(&T_THREE_DELTA, sizeof(double), 1, statFile);
  fread(&T_MAX_LENGTH, sizeof(double), 1, statFile);
  fread(&checkFreq, sizeof(int), 1, statFile);
  fread(&trainingSetSize, sizeof(int), 1, statFile);
  fread(&controlSetSize, sizeof(int), 1, statFile);

  printf("Training set size is %d, control set %d.\n\n",
	 trainingSetSize, controlSetSize);
  
  /* allocate arrays */
  tErrorBuckets=(double *)malloc(T_MAX_LENGTH*T_MAX_LENGTH*sizeof(double));
  tLengthBuckets=(int *)malloc(T_MAX_LENGTH*T_MAX_LENGTH*sizeof(int));
  cErrorBuckets=(double *)malloc(T_MAX_LENGTH*T_MAX_LENGTH*sizeof(double));
  cLengthBuckets=(int *)malloc(T_MAX_LENGTH*T_MAX_LENGTH*sizeof(int));

  /* read in the length buffer data */
  fread(tLengthBuckets, sizeof(int), T_MAX_LENGTH*T_MAX_LENGTH, statFile);
  fread(cLengthBuckets, sizeof(int), T_MAX_LENGTH*T_MAX_LENGTH, statFile);
  
  /* print the run parameters */
  printf("Training Tournament Auto Rater with:\n\t%s=%4.3g\tlrate=%4.3g\t"
	 "momentum=%4.3g\tseed=%4.3ld\n\tUsing %s network output mode.\n\n",
	 (stopType==AVG ? "avgErr" : "maxErr"), maxErr, lrate,
	 momentum, seedVal, (outputMode==BINARY ? "binary" : "complex"));

  /* initialize weights and biases */
  initNetVals();

  /* Now read from the stat file until it ends */
  do {

    /*
     * Read info from the stat file
     */
    
    /* iteration number */
    fread(&iterations, sizeof(int), 1, statFile);
    
    /* training set stats */
    fread(&tAvgSumSqrErr, sizeof(double), 1, statFile);
    fread(&tMinSumSqrErr, sizeof(double), 1, statFile);
    fread(&tMaxSumSqrErr, sizeof(double), 1, statFile);
    fread(&tPercentCorrect, sizeof(double), 1, statFile);
    fread(tErrorBuckets, sizeof(double), T_MAX_LENGTH*T_MAX_LENGTH, statFile);
    
    fread(tErrorDistrib, sizeof(int), 100, statFile);
    
    /* control set stats */
    fread(&cAvgSumSqrErr, sizeof(double), 1, statFile);
    fread(&cMinSumSqrErr, sizeof(double), 1, statFile);
    fread(&cMaxSumSqrErr, sizeof(double), 1, statFile);
    fread(&cPercentCorrect, sizeof(double), 1, statFile);
    fread(cErrorBuckets, sizeof(double), T_MAX_LENGTH*T_MAX_LENGTH, statFile);
    
    fread(cErrorDistrib, sizeof(int), 100, statFile);
    
    /* now write out the weights and biases */
    if(iterations==plotIteration && !plotMade && writeWeights)
      freadNetState(statFile, 1);
    else
      freadNetState(statFile, 0);
    
    /* print iteration information */
    if(!quietMode)
      printf("Iteration %d:\t\n"
	   "  Training Set:\n"
	   "\tSum Sqr Error (Avg, Max, Min):\t% 4.3g % 4.3g % 4.3g\n"
	   "\tPercent Correct:\t\t%4.3g\n"
	   "  Control Set:\n"
	   "\tSum Sqr Error (Avg, Max, Min):\t% 4.3g % 4.3g % 4.3g\n"
	   "\tPercent Correct:\t\t%4.3g\n\n",
	   iterations, tAvgSumSqrErr, tMaxSumSqrErr, tMinSumSqrErr,
	   tPercentCorrect*100, cAvgSumSqrErr, cMaxSumSqrErr, cMinSumSqrErr,
	   cPercentCorrect*100);

        /* write to appropriate output files */
    switch(plotType) {
      
    case NONE: break;

    case T_SUM_ERRORS:
      fprintf(dataFile, "%d\t%g\t%g\t%g\n", iterations, tMaxSumSqrErr,
	      tAvgSumSqrErr, tMinSumSqrErr);
      break;

    case C_SUM_ERRORS:
      fprintf(dataFile, "%d\t%g\t%g\t%g\n", iterations, cMaxSumSqrErr,
	      cAvgSumSqrErr, cMinSumSqrErr);
      break;

    case T_VS_C_SUM_ERRORS:
      fprintf(dataFile, "%d\t%g\t%g\n", iterations, tAvgSumSqrErr,
	      cAvgSumSqrErr);
      break;

    case T_VS_C_PERCENT_CORRECT:
      fprintf(dataFile, "%d\t%g\t%g\n", iterations, tPercentCorrect,
	      cPercentCorrect);
      break;

    case T_LENGTH_HISTOGRAM:
      if(iterations==plotIteration && !plotMade++) {
	lastGood=0;
	for(i=0;i<T_MAX_LENGTH;i++) {
	  for(j=0;j<T_MAX_LENGTH;j++) {
	    if(tLengthBuckets[(int)T_MAX_LENGTH*i+j]!=0) {
	      lastGood=tErrorBuckets[(int)T_MAX_LENGTH*i+j]/
		      (double)tLengthBuckets[(int)T_MAX_LENGTH*i+j];
	    }
	    fprintf(dataFile, "%g\n", lastGood);
	  }
	  fprintf(dataFile, "\n");
	}
      }
      break;

    case C_LENGTH_HISTOGRAM:
      if(iterations==plotIteration && !plotMade++) {
	lastGood=0;
	for(i=0;i<T_MAX_LENGTH;i++) {
	  for(j=0;j<T_MAX_LENGTH;j++) {
	    if(cLengthBuckets[(int)T_MAX_LENGTH*i+j]!=0) {
	      lastGood=cErrorBuckets[(int)T_MAX_LENGTH*i+j]/
		      (double)cLengthBuckets[(int)T_MAX_LENGTH*i+j];
	    }
	    fprintf(dataFile, "%g\n", lastGood);
	  }
	  fprintf(dataFile, "\n");
	}
      }
      break;

    case T_ERROR_DISTRIB:
      if(iterations==plotIteration && !plotMade++)
	for(i=0;i<100;i++)
	  fprintf(dataFile, "%d\t%g\n", i,
		  (double)tErrorDistrib[i]/(double)trainingSetSize);   
      break;

    case C_ERROR_DISTRIB:
      if(iterations==plotIteration && !plotMade++)
	for(i=0;i<100;i++)
	  fprintf(dataFile, "%d\t%g\n", i,
		  (double)cErrorDistrib[i]/(double)controlSetSize);   
      break;
      
    default: printf("Unknown plot type!\n\n");

    }

  } while(!feof(statFile));

  /* print how many iterations in the file */
  printf("Processed %d learning cycles.\n\n", iterations);
  
  /* free buffers and close files */
  free(tErrorBuckets);
  free(tLengthBuckets);
  free(cErrorBuckets);
  free(cLengthBuckets);

  free(aOneWeights);
  free(aTwoWeights);
  free(aThreeWeights);
  free(aOutputWeights);

  free(bOneWeights);
  free(bTwoWeights);
  free(bThreeWeights);
  free(bOutputWeights);

  fclose(statFile);
  
  return 0;
  
}





