#include <stdio.h>
#include <string.h>

typedef struct {

  int ind1;
  int ind2;
  int result;
  
} Comparison;

int main ( int argc, char **argv ) {

  int i,j,k,m;
  int numSeqs;
  int maxGen, maxInd;
  FILE *comparisonLog;
  FILE **outFiles;        /* array of output files */
  int currentOutFile;    /* points to the output file for the next ind */
  int *fileNumInds;      /* stores the number of inds in each output file */
  int **fileSequences;
  size_t temp;
  char tempString[100];
  int splitNum;
  char **noteSequenceArray;
  Comparison *compArray;
  int compArrayLength=0;
  int compArrayPos=0;
  int compResult;
  
  /* open the comparison file */
  comparisonLog=fopen(argv[2],"r");

  /* find the number of pieces to split into */
  splitNum=atoi(argv[1]);
  
  /* get the number of individuals and generations */
  fread(&maxGen, sizeof(int), 1, comparisonLog);
  fread(&maxInd, sizeof(int), 1, comparisonLog);  

  /* open the output files */
  outFiles=(FILE **)malloc(splitNum*sizeof(FILE *));
  fileNumInds=(int *)malloc(splitNum*sizeof(int));
  fileSequences=(int **)malloc(splitNum*sizeof(int *));
  for(i=0;i<splitNum;i++) {
    sprintf(tempString,"%d%s",i,argv[2]);
    outFiles[i]=fopen(tempString,"w+");
    fwrite(&i, sizeof(int), 1, outFiles[i]); /* make space for an int */
    fileNumInds[i]=0;
    fileSequences[i]=(int *)malloc(((maxGen+1)*maxInd+1)*sizeof(int));
    for(j=0;j<(maxGen+1)*maxInd+1;j++)
      (fileSequences[i])[j]=-1;
  }
  currentOutFile=0;

  /* print blurb */
  printf("Splitting GP-Music Comparisons into %d parts\n\nInput file has "
	 "%d generations with %d "
	 "individuals per generation\n\n", splitNum, maxGen, maxInd);

  /* allocate arrays */
  noteSequenceArray=(char **)malloc((maxGen+1)*maxInd*sizeof(char *));
  compArray=(Comparison *)malloc((maxGen+1)*maxInd*sizeof(Comparison));
  compArrayLength=(maxGen+1)*maxInd;
		   
  /* now loop through generations and individuals */
  for(i=0;i<=maxGen;i++) {

    /* get the noteSequences for this generation */
    for(j=0;j<maxInd;j++) {

      /* read size of sequence */
      fread(&temp, sizeof(size_t), 1, comparisonLog);

      /* allocate and read note sequence */
      noteSequenceArray[i*maxInd+j]=(char *)malloc(temp*sizeof(char));
      fread(noteSequenceArray[i*maxInd+j], sizeof(char), temp, comparisonLog);

      /* skip rating */
      fseek(comparisonLog, sizeof(double), SEEK_CUR);

    }

    /* now read the comparisons for this generation */
    for(k=0;k<maxInd;k++) 
      for(m=0;m<maxInd;m++) {
	fread(&compResult, sizeof(int), 1, comparisonLog);
	if(compResult) {
	  if(compArrayLength<=compArrayPos) {
	    compArrayLength*=2;
	    compArray=(Comparison *)realloc(compArray,
			  compArrayLength*sizeof(Comparison));
	  }
	  compArray[compArrayPos].ind1=i*maxInd+k;
	  compArray[compArrayPos].ind2=i*maxInd+m;
	  compArray[compArrayPos++].result=compResult;
	}
      }
  }

  /* split up the comparisons and write them to the files */
  for(i=0;i<compArrayPos;i++) {

    /* see if the first noteSequence of the comparison is slated for
       this output file yet, if not put it there. */
    for(j=0;(fileSequences[currentOutFile])[j]!=-1;j++)
      if((fileSequences[currentOutFile])[j]==compArray[i].ind1) break;

    if((fileSequences[currentOutFile])[j]==-1)
      (fileSequences[currentOutFile])[j]=compArray[i].ind1;

    /* adjust index of individual 1 to index for this output file */
    compArray[i].ind1=j;
      
    /* see if the second noteSequence of the comparison is slated for
       this output file yet, if not put it there. */
    for(j=0;(fileSequences[currentOutFile])[j]!=-1;j++)
      if((fileSequences[currentOutFile])[j]==compArray[i].ind2) break;

    if((fileSequences[currentOutFile])[j]==-1)
      (fileSequences[currentOutFile])[j]=compArray[i].ind2;

    /* adjust index of individual 1 to index for this output file */
    compArray[i].ind2=j;
             
    fwrite(&compArray[i], sizeof(Comparison), 1, outFiles[currentOutFile]);
    
    /* change to the next output file and increment fileNumInds */
    fileNumInds[currentOutFile]++;
    currentOutFile=(currentOutFile+1)%splitNum;
    
  }

  /* write each files noteSequences */
  for(i=0;i<splitNum;i++) {

    /* count the number of note sequences for this file */
    for(numSeqs=0;(fileSequences[i])[numSeqs]!=-1;numSeqs++);
    
    /* write the number to the file */
    fwrite(&numSeqs, sizeof(int), 1, outFiles[i]);

    /* now write the note sequences */
    for(j=0;j<numSeqs;j++) {

      /* figure out the size of the sequence and write it */
      temp=strlen(noteSequenceArray[(fileSequences[i])[j]])+1;
      fwrite(&temp, sizeof(size_t), 1, outFiles[i]);

      /* write note sequence */
      fwrite(noteSequenceArray[(fileSequences[i])[j]], sizeof(char),
	     temp, outFiles[i]);

    }

  }
  
  /* close the rating file */
  fclose(comparisonLog);

  /* write the number of individuals into each output file and close them */
  for(i=0;i<splitNum;i++) {
    fseek(outFiles[i], 0, SEEK_SET);
    fwrite(&(fileNumInds[i]), sizeof(int), 1, outFiles[i]);
    fclose(outFiles[i]);
  }
  
  /* deallocate arrays */
  free(outFiles);
  free(fileNumInds);
  free(compArray);
  for(i=0;i<(maxGen+1)*maxInd;i++)
    free(noteSequenceArray[i]);
  free(noteSequenceArray);
  exit(0);
  
}


