#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "xm_lib.h"

#ifdef GP_MUSIC

#include "lilgp.h"

#endif

#ifndef MALLOC

#define MALLOC malloc

#endif

#ifndef REALLOC

#define REALLOC realloc

#endif

#ifndef FREE

#define FREE free

#endif


/*
 *   fwriteXMPatternNoteData - writes a pattern of note data from the notes
 *      contained in noteString to xmFile.  The note values written have
 *      values from 1-72, 1 being C-0, and 88 signaling no note.  Only
 *      channel 0 has notes.  Notes below 35 use instrument 2, notes
 *      above use instrument 1.  Four channels
 *      are used. The returned value is the size of the packed data.
 */
short int 
fwriteXMPatternNoteData(FILE *xmFile, char *noteString) {

  short int counter=0;
  int i;
  int rows;
  char tempChar=0x80;
  char note;
  char bytes[3];

  /* initialize bytes array */
  for(i=0;i<3;i++)
    bytes[i]=0x80;

  /* find the number of rows */
  rows=(int)strlen(noteString);
  
  /* loop through rows */
  for(i=0;i<rows;i++) {

    note=noteString[i];
    
    /* decide if this is a note */
    if(note != 88) {

      /* write start byte */
      tempChar=0x83;
      fwrite(&tempChar, sizeof(char), 1, xmFile);

      /* write note */
      note--;
      fwrite(&note, sizeof(char), 1, xmFile);

      /* write instrument */
      if(note<35) tempChar=0x02;
      else tempChar=0x01;
      fwrite(&tempChar, sizeof(char), 1, xmFile);

      counter+=3;
      
    } else {

      /* no note here, so print place holder */
      tempChar=0x80;
      fwrite(&tempChar, sizeof(char), 1, xmFile);
      counter++;
      
    }

    /* write the rest of the channels notes (nothing) */
    fwrite(&bytes, sizeof(char), 3, xmFile);
    counter+=3;

  }

  return counter;

} /* end of fwriteXMPatternNoteData */

/*
 *   fwriteXMPatternInfo - writes one pattern worth of data to xmFile. All
 *      notes are in column 0. Notes below 35 use instrument 2, notes
 *      above use instrument 1.  4 channels are used,
 *      and as many pattern rows as there are items in the noteString
 *      which contains the notse for the pattern.
 */
void
fwriteXMPatternInfo(FILE *xmFile, char *noteString)
{

  int tempInt;
  char tempChar;
  short int tempShort;
  long dataSizePos;
  
  /* Set pattern header length */
  tempInt=9;
  fwrite(&tempInt, sizeof(int), 1, xmFile);

  /* Set packing type (always 0) */
  tempChar=0;
  fwrite(&tempChar, sizeof(char), 1, xmFile);
  
  /* Set number of rows in pattern */
  tempShort=(short)strlen(noteString);
  fwrite(&tempShort, sizeof(short int), 1, xmFile);
    
  /* Store position for pattern data size */
  dataSizePos=ftell(xmFile);
  fseek(xmFile,2,SEEK_CUR);

  /* Write random pattern data */
  tempShort=fwriteXMPatternNoteData(xmFile, noteString);
  
  /* Now store Data Size */
  fseek(xmFile,dataSizePos, SEEK_SET);
  fwrite(&tempShort, sizeof(short int), 1, xmFile);
  
  return;
  
} /* end of fwriteXMPatternInfo */

/*
 *   fprintXMHeaderInfo - prints the header info for a XM file to the given
 *       output file.  At the end the file pointer will be set to the
 *       beginning of the pattern data.  Patterns is returned with the number
 *       of patterns. Instruments is returned with the number of instruments.
 *       Returns '1' if this is not a XM file.
 */
int
fprintXMHeaderInfo(FILE *xmFile, FILE *outFile,
		   int *patterns, int *instruments, int *channels)
{

  char tempString[23];
  short int tempShort=0;
  int headerSize=0;
  
  /* seek to start of XM file */
  fseek(xmFile,0,SEEK_SET);

  /* check to see if it is really a valid XM file */
  tempString[17]=0;
  
  fread(tempString, sizeof(char), 17, xmFile);

  fprintf(outFile, "Cookie =\t\t'%s'\n", tempString);
  
  if (strcmp(tempString, "Extended Module: "))
    return 1;
  
  /* Now get Module Name */
  tempString[20]=0;

  fread(tempString, sizeof(char), 20, xmFile);

  fprintf(outFile, "Module Name =\t\t'%s'\n", tempString);

  /* Now get the tracker name */
  fseek(xmFile,1,SEEK_CUR);

  tempString[20]=0;

  fread(tempString, sizeof(char), 20, xmFile);

  fprintf(outFile, "Tracker Name =\t\t'%s'\n", tempString);

  /* Get version number */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "Version Number =\t%#hx\n", tempShort);

  /* Get header size */
  fread(&headerSize, sizeof(int), 1, xmFile);

  fprintf(outFile, "Header Size =\t\t%d bytes\n", headerSize);

  /* Get song length */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "Song Length =\t\t%hd pattern\n", tempShort);

  /* Get restart position */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "Restart Position =\tPattern %hd\n", tempShort);

  /* Get number channels */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "Channels =\t\t%hd\n", tempShort);

  *channels=(int)tempShort;
  
  /* Get number of patterns */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "Patterns =\t\t%hd\n", tempShort);

  *patterns=(int)tempShort;
  
  /* Get number of instruments */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "Instruments =\t\t%hd\n", tempShort);

  *instruments=(int)tempShort;
  
  /* Get Frequency Table information */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  if ( 0x0100 & tempShort )
    fprintf(outFile, "Frequency Table =\tAmiga\n");
  else
    fprintf(outFile, "Frequency Table =\tLinear\n");
  
  /* Get default tempo */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "Default Tempo =\t\t%hd\n", tempShort);
  
  /* Get default beats per minute */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "Default BPM =\t\t%hd\n\n", tempShort);

  /* seek to beginning of pattern data */
  fseek(xmFile,60+headerSize,SEEK_SET);
  
  return 0;

} /* end of fprintXMHeaderInfo */

/*
 *   sprintNote - converts note from its numeric format to 'C#0' type
 *      format, and sprints it to outString.
 */
void
sprintNote(char *outString, int note) {

  int octave;
  char noteLetter;
  char sharpLetter;

  /* if note out of range, print value and return */
  if(note>72 || note<1) {  
    sprintf(outString,"%03d", note); 
    return;
  }
  
  /* figure out the octave */
  octave=(note+3)/12;

  /* now figure out which note in the octave */
  note=note%12;

  switch(note) {

  case  0:    noteLetter='C'; sharpLetter='-'; break;
  case  1:    noteLetter='C'; sharpLetter='#'; break;
  case  2:    noteLetter='D'; sharpLetter='-'; break;
  case  3:    noteLetter='D'; sharpLetter='#'; break;
  case  4:    noteLetter='E'; sharpLetter='-'; break;
  case  5:    noteLetter='F'; sharpLetter='-'; break;
  case  6:    noteLetter='F'; sharpLetter='#'; break;
  case  7:    noteLetter='G'; sharpLetter='-'; break;
  case  8:    noteLetter='G'; sharpLetter='#'; break;
  case  9:    noteLetter='A'; sharpLetter='-'; break;
  case 10:    noteLetter='A'; sharpLetter='#'; break;
  case 11:    noteLetter='B'; sharpLetter='-'; break;
  default:    noteLetter='-'; sharpLetter='-'; break;
      
  }

  /* Now print the note */
  sprintf(outString,"%c%c%d", noteLetter, sharpLetter, octave); 
  
  return;
  
  
} /* end of sprintNote */

/*
 *   fprintNote - converts note from its numeric format to 'C#0' type
 *      format, and prints it to outfile followed by two spaces.
 */
void
fprintNote(int note, FILE *outFile) {

  int octave;
  char noteLetter;
  char sharpLetter;

  /* if note out of range, print value and return */
  if(note>72 || note<1) {
    fprintf(outFile,"%03d  ", note); 
    return;
  }
  
  /* figure out the octave */
  octave=(note+3)/12;
  
  /* now figure out which note in the octave */
  note=note%12;

  switch(note) {

  case  0:    noteLetter='C'; sharpLetter='-'; break;
  case  1:    noteLetter='C'; sharpLetter='#'; break;
  case  2:    noteLetter='D'; sharpLetter='-'; break;
  case  3:    noteLetter='D'; sharpLetter='#'; break;
  case  4:    noteLetter='E'; sharpLetter='-'; break;
  case  5:    noteLetter='F'; sharpLetter='-'; break;
  case  6:    noteLetter='F'; sharpLetter='#'; break;
  case  7:    noteLetter='G'; sharpLetter='-'; break;
  case  8:    noteLetter='G'; sharpLetter='#'; break;
  case  9:    noteLetter='A'; sharpLetter='-'; break;
  case 10:    noteLetter='A'; sharpLetter='#'; break;
  case 11:    noteLetter='B'; sharpLetter='-'; break;
  default:    noteLetter='-'; sharpLetter='-'; break;
      
  }

  /* Now print the note */
  fprintf(outFile,"%c%c%d  ", noteLetter, sharpLetter, octave); 
  
  return;
  
  
} /* end of fprintNote */

/*
 *   sprintNoteSequence - creates a note sequence string into a human readable
 *      string with six notes per line, suitable for display in the list
 *      rater form and returns it.  
 */
char *
sprintNoteSequence(const char *noteSequence)
{

  int i,j;
  int k;  /* keeps track of current pos in newString */
  int seqLength;
  char *newString;
  
  /* find the length of the noteSequence */
  seqLength=strlen(noteSequence);

  /* realloc newString to fit the new sequence */
  newString=(char *)MALLOC((seqLength+1)*6*sizeof(char));
  
  /* now write the new note sequence */
  j=0;
  k=0;
  
  for(i=0;i<seqLength;i++) {

    sprintf(newString+k++, "(");

    /* print either pause or note */
    if( noteSequence[i] == 88 )
      sprintf(newString+k,"   ");
    else
      sprintNote(newString+k, (int)(noteSequence[i]-1));

    k+=3;

    sprintf(newString+k++,")");
    
    if(++j % 6 == 0)
      sprintf(newString+k++,"\n");
    else
      sprintf(newString+k++," ");
    
  }

  return newString;
  
} /* end of sprintNoteSequence */

/*
 *   fprintXMPatternNoteData - prints one patterns note data in xmFile to
 *      outFile. The size of the data is expected to be dataSize bytes, and
 *      the XM should have 'channels' channels, and 'rows' rows in this
 *      pattern. Expects xmFile to be queued to the beginning of note data,
 *      and leaves xmFile queued to the beginning of the next
 *      pattern/instrument.
 */
void 
fprintXMPatternNoteData(FILE *xmFile, FILE *outFile, int dataSize,
			int rows, int channels) {

  int i, j;
  char tempChar;
  char firstByte;
  char bytes[4];
  
  /* loop through rows and channels */
  for(i=0;i<rows;i++) {

    /* Start printing line */
    fprintf(outFile, "\t");
        
    for(j=0;j<channels;j++) {

      /* read first byte of this 'note' */
      fread(&firstByte, sizeof(char), 1, xmFile);
      
      /* check for packing */
      if(0x80 & firstByte) {
	
	/* see if there is any data for this note */
	if(0x1F & firstByte) {

	  /* Go through and get each byte that is needed */

	  /* get note if needed */
	  if (0x01 & firstByte) {
	    
	    fread(&tempChar, sizeof(char), 1, xmFile);

	    fprintNote((int)tempChar, outFile);
	    
	  }

	  /* get instrument if needed */
	  if (0x02 & firstByte ) fread(&tempChar, sizeof(char), 1, xmFile);

	  /* get volume column if needed */
	  if (0x04 & firstByte ) fread(&tempChar, sizeof(char), 1, xmFile);

	  /* get effect type if needed */
	  if (0x08 & firstByte ) fread(&tempChar, sizeof(char), 1, xmFile);

	  /* get effect parameter if needed */
	  if (0x10 & firstByte ) fread(&tempChar, sizeof(char), 1, xmFile);
	  	  
	} else {

	  /* no note here, so print place holder */
	  fprintf(outFile, "---  ");
	  
	}
	
      } else {

	/* Print this note */
	fprintNote((int)firstByte, outFile);
	
	/* need to read next four bytes */
	fread(bytes, sizeof(char), 4, xmFile);
	
      }

    }

    /* Go to next line */
    fprintf(outFile, "\n");
      
  }

  /* Go to next line */
  fprintf(outFile, "\n");
  
  return;
  
} /* end of fprintXMPatternNoteData */

/*
 *   fprintXMPatternInfo - prints one patterns data in xmFile to outFile.
 *      Expects xmFile to be queued to the beginning of pattern info, and
 *      leaves xmFile queued to the beginning of the note data. Channels is
 *      the number of channels in this XM file.  If printNoteData is true,
 *      the note information is also printed.
 */
void
fprintXMPatternInfo(FILE *xmFile, FILE *outFile,
		    int channels, int printNoteData)
{

  int headerSize;
  short int dataSize;
  long patternStart=0;
  short int rows;

  /* find pattern start */
  patternStart=ftell(xmFile);
  
  /* Get pattern header length */
  fread(&headerSize, sizeof(int), 1, xmFile);

  fprintf(outFile, "\tHeader Size =\t%d\n", headerSize);

  fseek(xmFile, 1, SEEK_CUR);  /* skip packing type */

  /* Get number of rows in pattern */
  fread(&rows, sizeof(short int), 1, xmFile);

  fprintf(outFile, "\tRows =\t\t%hd\n", rows);
    
  /* Get pattern data size */
  fread(&dataSize, sizeof(short int), 1, xmFile);

  fprintf(outFile, "\tData Size =\t%hd\n\n", dataSize);

  
  /* If requested, print pattern note info */
  if(printNoteData) {

    fprintf(outFile, "\tPattern Note Data:\n\n");
    
    fprintXMPatternNoteData(xmFile, outFile, dataSize, (int)rows, channels);

  }
  
  /* Go to end of this pattern */
  fseek(xmFile, patternStart+headerSize+dataSize, SEEK_SET);
  
  return;
  
} /* end of fprintXMPatternInfo */

/*
 *   fprintXMExtendedInstrumentInfo - prints one instruments extended data,
 *      including details and sample information, in xmFile to
 *      outFile.  Expects xmFile to be queued to the beginning of extended
 *      info (sample header size in xm.txt).  It returns the size of the
 *      sample headers, and queues the file to the beginning of the first
 *      sample for this instrument.
 */
int
fprintXMExtendedInstrumentInfo(FILE *xmFile, FILE *outFile) {

  short int tempShort;
  char tempChar;
  int headerSize;
  long extendedStart=0;
    
  /* find extended info start */
  extendedStart=ftell(xmFile);
  
  /* Get sample header size */
  fread(&headerSize, sizeof(int), 1, xmFile);

  fprintf(outFile, "\tSample Header Size =\t%d\n", headerSize);

  fseek(xmFile, 192, SEEK_CUR);  /* skip notes, panning and volume pts */

  /* Get number of volume points */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tVolume points =\t\t%hd\n", tempChar);
  
  /* Get number of panning points */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tPanning points =\t%hd\n\n", tempChar);

  /* Get volume sustain point */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tVolume Sustain Point =\t%hd\n", tempChar);

  /* Get volume loop start point */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tVolume Loop Start =\t%hd\n", tempChar);

  /* Get volume loop end */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tVolume Loop End =\t%hd\n\n", tempChar);

  /* Get panning sustain point */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tPanning Sustain Point =\t%hd\n", tempChar);

  /* Get panning loop start point */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tPanning Loop Start =\t%hd\n", tempChar);

  /* Get panning loop end */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tPanning Loop End =\t%hd\n\n", tempChar);

  /* Get volume type */
  fread(&tempChar, sizeof(char), 1, xmFile);

  if(tempChar & 0x1)
    fprintf(outFile, "\tVolume is On\n");
  else
    fprintf(outFile, "\tVolume is Off\n");

  if(tempChar & 0x2)
    fprintf(outFile, "\tVolume Sustains\n");
  else
    fprintf(outFile, "\tVolume Doesn't Sustain\n");

  if(tempChar & 0x4)
    fprintf(outFile, "\tVolume Loops\n\n");
  else
    fprintf(outFile, "\tVolume Doesn't Loop\n\n");

  /* Get panning type */
  fread(&tempChar, sizeof(char), 1, xmFile);

  if(tempChar & 0x1)
    fprintf(outFile, "\tPanning is On\n");
  else
    fprintf(outFile, "\tPanning is Off\n");

  if(tempChar & 0x2)
    fprintf(outFile, "\tPanning Sustains\n");
  else
    fprintf(outFile, "\tPanning Doesn't Sustain\n");

  if(tempChar & 0x4)
    fprintf(outFile, "\tPanning Loops\n\n");
  else
    fprintf(outFile, "\tPanning Doesn't Loop\n\n");

  /* Get Vibrato Type */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tVibrato Type =\t\t%hd\n", tempChar);

  /* Get Vibrato Sweep */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tVibrato Sweep =\t\t%hd\n", tempChar);

  /* Get Vibrato Depth */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tVibrato Depth =\t\t%hd\n", tempChar);

  /* Get Vibrato Rate */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\tVibrato Rate =\t\t%hd\n\n", tempChar);

  /* Get Volume Fadeout */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "\tVolume Fadeout =\t%hd\n\n", tempShort);

  fseek(xmFile, 2, SEEK_CUR);  /* reserved section */
  
  return headerSize;

} /* end of fprintXMExtendedInstrumentInfo */

/*
 *   fprintXMSampleInfo - prints one samples info in xmFile to
 *      outFile.  Expects xmFile to be queued to the beginning of sample
 *      info, and queues xmFile to the beginning of the next sample/instrument
 *      when done. headerSize is the size of the sample header, and is
 *      used to skip the file to the next sample header.  The returned
 *      value is the length of the sample data.
 */
int
fprintXMSampleInfo(FILE *xmFile, FILE *outFile, int headerSize) {

  char tempChar;
  int tempInt;
  int dataSize;
  long sampleStart=0;
  char tempString[23];
  
  /* find pattern start */
  sampleStart=ftell(xmFile);
  
  /* Get sample length */
  fread(&dataSize, sizeof(int), 1, xmFile);

  fprintf(outFile, "\t\tSample Length =\t\t%d\n", dataSize);

  /* Get sample loop start */
  fread(&tempInt, sizeof(int), 1, xmFile);

  fprintf(outFile, "\t\tSample Loop Start =\t%d\n", tempInt);
    
  /* Get sample loop length */
  fread(&tempInt, sizeof(int), 1, xmFile);

  fprintf(outFile, "\t\tSample Loop Length =\t%d\n", tempInt);

  /* Get sample volume */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\t\tSample Volume =\t\t%hd\n", tempChar);

  /* Get sample finetune */
  fread(&tempChar, sizeof(char), 1, xmFile);
  
  fprintf(outFile, "\t\tSample Finetune Value =\t%hd\n", tempChar);

  /* Get sample type */
  fread(&tempChar, sizeof(char), 1, xmFile);

  switch(tempChar & 0x3) {
  case 0:
    fprintf(outFile, "\t\tSample has No Loop\n");
    break;
  case 1:
    fprintf(outFile, "\t\tSample has Forward Loop\n");
    break;
  case 2:
    fprintf(outFile, "\t\tSample has Ping-Pong Loop\n");
    break;
  }

  if(tempChar & 0x8)
    fprintf(outFile, "\t\tSample has 16-bit data\n");
  else
    fprintf(outFile, "\t\tSample has 8-bit data\n");

  /* Get Sample Panning */
  fread(&tempChar, sizeof(char), 1, xmFile);
  
  tempInt=(int)tempChar;
  tempInt+=128;

  fprintf(outFile, "\t\tSample Panning Value =\t%d\n", tempInt);

  /* Get Sample Relative Note */
  fread(&tempChar, sizeof(char), 1, xmFile);

  fprintf(outFile, "\t\tSample Relative Note =\t%hd\n", tempChar);

  fseek(xmFile,1, SEEK_CUR); /* seek passed reserved space */
  
  /* Get sample name */
  tempString[22]=0;

  fread(tempString, sizeof(char), 22, xmFile);

  fprintf(outFile, "\t\tSample Name =\t\t'%s'\n\n", tempString);
  
  /* for debugging *//*
  fprintf(outFile, "Current offset is %#lx\n\n", ftell(xmFile));
  fprintf(outFile, "Expect next sample at: %#lx\n\n",
	  sampleStart+headerSize+dataSize); */
  
  /* seek to beginning of next sample/instrument */
  fseek(xmFile,sampleStart+headerSize,SEEK_SET);
  
  return dataSize;

} /* end of fprintXMSampleInfo */

/*
 *   fprintXMInstrumentInfo - prints one instruments data in xmFile to
 *      outFile.  Expects xmFile to be queued to the beginning of instrument
 *      info, and queues xmFile to the beginning of the next instrument/end
 *      of file when done.
 */
void
fprintXMInstrumentInfo(FILE *xmFile, FILE *outFile) {

  short int tempShort;
  int instrumentSize;
  int sampHeaderSize;
  int dataSize=0;
  long instrumentStart=0;
  char tempString[23];
  int i;
  
  /* find instrument start */
  instrumentStart=ftell(xmFile);
  
  /* Get instrument size */
  fread(&instrumentSize, sizeof(int), 1, xmFile);

  fprintf(outFile, "\tInstrument Size =\t%d\n", instrumentSize);

  /* Get instrument name */
  tempString[22]=0;

  fread(tempString, sizeof(char), 22, xmFile);

  fprintf(outFile, "\tInstrument Name =\t'%s'\n", tempString);

  fseek(xmFile, 1, SEEK_CUR);  /* skip instrument type */
  
  /* Get number of samples in instrument */
  fread(&tempShort, sizeof(short int), 1, xmFile);

  fprintf(outFile, "\tNumber of Samples =\t%hd\n\n", tempShort);

  if(tempShort >0) {

    sampHeaderSize = fprintXMExtendedInstrumentInfo(xmFile, outFile);

    /* go to start of samples */
    fseek(xmFile,instrumentStart+instrumentSize,SEEK_SET);
    
    /* Print out each samples information */
    for(i=0;i<tempShort;i++) {

      fprintf(outFile, "\tSample %d Information:\n", i);

      dataSize+=fprintXMSampleInfo(xmFile, outFile, sampHeaderSize);
      
    }

    /* seek over sample data */
    fseek(xmFile,dataSize,SEEK_CUR);
      
  } else {
  
    /* seek to beginning of next instrument/end of file */
    fseek(xmFile,instrumentStart+instrumentSize,SEEK_SET);

  }
  
  return;

} /* end of fprintXMInstrumentInfo */






